#  spec file for canrestbus library
#
#  Copyright (c) 2019  Andreas Stewering-Bone <ab@igh.de>
#
#  This file is part of the canrestbus library.
#
#  The canrestbus library is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or (at your
#  option) any later version.
#
#  The canrestbus library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
#  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
#  License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with the canrestbus library. If not, see <http://www.gnu.org/licenses/>.
#
#

%define pkgname canrestbus
%define libname lib%{pkgname}2_1

Name:    canrestbus
Version: 2.1.0
Release: 1
Summary: CAN Restbus simulation library
Group:   System/Libraries
License: GPL-2.0
Url:     http://www.etherlab.org/
Source: canrestbus_2.1.0.orig.tar.bz2
Prereq: /sbin/ldconfig



BuildRequires: gcc gcc-c++ coreutils pcre-devel
BuildRequires: cmake
BuildRequires: mercurial
BuildRequires: pkgconfig

%description
CAN Restbus simulation library.


#----------------------------------------------------------------------------
# Build step
# This must be before packaging, because otherwise the version number
# changes
#----------------------------------------------------------------------------
%prep
%setup -n %{pkgname}_%{version}.orig

%build
%cmake --no-warn-unused-cli \
       -DCMAKE_INSTALL_SYSCONFDIR=%{_sysconfdir} \
       -DCMAKE_INSTALL_INCLUDEDIR=%{_includedir} \
       -DCMAKE_INSTALL_BINDIR=%{_bindir} \
       -DCMAKE_INSTALL_LIBDIR=%{_libdir} \
       -DBUILDTOOLS=1 \
       -DMATLAB=1

%if 0%{?make_jobs:1}
%make_jobs
%else
%__make %{?_smp_mflags} VERBOSE=1
%endif


# put a link to current build directory in source tree
%__ln_s -f $PWD %{_builddir}/%{buildsubdir}/_top_builddir

%install
%if 0%{?cmake_install:1}
%cmake_install
%else
DESTDIR=%{buildroot} %__make install -C _top_builddir
%endif


%package -n %{libname}
Summary:    CAN Restbus Library
Group:      System/Libraries
Provides:   %{libname} = %{version}

%description -n %{libname}
CAN Restbus simulation library.


%files -n %{libname}
%defattr(-,root,root)
%{_libdir}/lib%{pkgname}.so.*

%package -n libcanrestbus-devel
Summary:    Development files for CAN Restbus Library
Group:      Development/Libraries/C and C++
Requires:   lib%{pkgname} = %{version}

%description -n libcanrestbus-devel
CAN Restbus simulation library, Develeopment files.

%files -n libcanrestbus-devel
%defattr(-,root,root)
%dir %{_includedir}/restbus
%{_includedir}/restbus/*.h
%{_libdir}/lib%{pkgname}.so
%{_libdir}/pkgconfig/lib%{pkgname}.pc

%package -n canrestbus-tools
Summary:    Tools based on CAN Restbus Library
Group:      Productivity/Scientific/Other
Requires:   lib%{pkgname} = %{version}

%description -n canrestbus-tools
CAN Restbus simulation library, User space tools.


%files -n canrestbus-tools
/usr/bin/can_decoder
/usr/bin/can_replay
/usr/bin/can_lsa2l


%package -n canrestbus-matlab
Summary:    Matlab tools based on CAN Restbus Library
Group:      Productivity/Scientific/Other
Requires:   lib%{pkgname} = %{version}

%description -n canrestbus-matlab
CAN Restbus simulation library, Matlab/Simulink/Codegeneration tools.


%files -n canrestbus-matlab
%dir /usr/share/canrestbus
/usr/share/canrestbus/*


%post -n %{libname} -p /sbin/ldconfig    
%postun -n %{libname} -p /sbin/ldconfig

%changelog
* Thu May 09 2019 ab@igh.de
- Created for use with OBS
