/*  QT Abstraction for Frame and Signal handling
    Copyright (C) 2018 Andreas Stewering-Bone

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QTimer>

#include "ui_MainWindow.h"
#include "QCANDecode.h"

/****************************************************************************/

class SignalEntry
{
 public:
  SignalEntry(QString name, double value = 0.0);
  ~SignalEntry();
  double getValue() const {return value;};
  QString getName() const {return name;};
  void setValue(double value);
 private:
  double value;
  QString name;
};

class SignalModel : public QAbstractTableModel
{
 private:
  QList<SignalEntry> m_data;
 public:
  SignalModel(QObject * parent = 0);
  void clear();
  int rowCount(const QModelIndex &) const;
  int columnCount(const QModelIndex &) const;
  QVariant data(const QModelIndex &index, int role) const;
  bool setData(const QModelIndex &index,
	       const QVariant &value, int role);
  QVariant headerData(int section, Qt::Orientation orientation, int role) const;
  Qt::ItemFlags flags(const QModelIndex &index) const;
  void append(const SignalEntry & signal);
  double getValue(QString name);
};

/****************************************************************************/

class MainWindow:
    public QMainWindow,
    public Ui::MainWindow
{
    Q_OBJECT

    public:
      MainWindow(QWidget * = 0);
      ~MainWindow();



    private:
      QCANSocketMessage *can;
      QStringList slist;
      bool started;
      SignalModel *smdl;
      QTimer *sendtimer;
    private slots:
      void start();
      void stop();
      void receive(unsigned int id, QDateTime &ts);
      void error(int error, QString errstr);
      void send();
};

/****************************************************************************/

#endif
