/*  Byte order utils
    Copyright (C) 2017 Andreas Stewering-Bone

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#include <byteswap.h>

#include "cs_byteorder.h"


/**************************************************************************/
cs_bo_t cs_bo_endianess()
/**************************************************************************/
{
    union { long l; char c[sizeof (long)]; } u;
    u.l = 1;
    if(u.c[sizeof (long) - 1] == 1)
        return BO_MOTOROLLA; //Big Endian, Motorola
    else
        return BO_INTEL; //Little Endian, Intel

}

/**************************************************************************/
uint16_t cs_bo_swap16(
		      cs_bo_t endianess,
		      uint16_t value
		      )
/**************************************************************************/
{
    uint16_t rv;
    cs_bo_t local = cs_bo_endianess();
    if(endianess != BO_UNKNOWN){
      if(local != endianess){
        //swap bytes
        rv = bswap_16(value);
        return rv;
      }
    }
    return value;
}

/**************************************************************************/
uint32_t cs_bo_swap32(
    		      cs_bo_t endianess,
		      uint32_t value
		      )
/**************************************************************************/
{
    uint32_t rv;
    cs_bo_t local = cs_bo_endianess();
    if(endianess != BO_UNKNOWN){
      if(local != endianess){
        //swap bytes
        rv = bswap_32(value);
        return rv;
      }
    }
    return value;
}

/**************************************************************************/
uint64_t cs_bo_swap64(
    		      cs_bo_t endianess,
		      uint64_t value
		      )
/**************************************************************************/
{
    uint64_t rv;
    cs_bo_t local = cs_bo_endianess();
    if(endianess != BO_UNKNOWN){
      if(local != endianess){
        //swap bytes
        rv = bswap_64(value);
        return rv;
      }
    }
    return value;
}
