/*  
    Copyright (C) 2017 Andreas Stewering-Bone

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. */

/**
 * @file cs_byteorder.h
 * @author Andreas Stewering-Bone
 * @date 9 Oct 2017
 * @brief File containing byte order utils.
 *
 * File containing byte order utils.
*/
#ifndef CS_BYTEORDER_H
#define CS_BYTEORDER_H

#include <inttypes.h>

/** \enum cs_bo_t
    Byteorder definition
*/
typedef enum{
  BO_UNKNOWN = 0x00,   /**< Not defined*/
  BO_INTEL = 0x01,     /**< Byteorder Intel, LSB*/
  BO_MOTOROLLA = 0x02  /**< Byteorder Motorolla, MSB*/
} cs_bo_t;

#ifdef __cplusplus
extern "C" {
#endif

/** Function detect endianess of local CPU
      \retval 0 Intel
      \retval 1 Motorola
  */
  cs_bo_t cs_bo_endianess();

  /** Function swap raw data to processor specific data, type uint16_t
      \retval swapped uint16_t
  */
  uint16_t cs_bo_swap16(
			cs_bo_t endianess,  /**< Remote ECU Byteorder*/
			uint16_t value      /**< Value to convert*/
			);

  /** Function swap raw data to processor specific data, type uint32_t
      \retval swapped uint32_t
  */
  uint32_t cs_bo_swap32(
			cs_bo_t endianess,    /**< Remote ECU Byteorder*/ 
			uint32_t value        /**< Value to convert*/
			);

  /** Function swap raw data to processor specific data, type uint64_t
      \retval swapped uint32_t
  */
  uint64_t cs_bo_swap64(
			cs_bo_t endianess,    /**< Remote ECU Byteorder*/
			uint64_t value        /**< Value to convert*/
			);

#ifdef __cplusplus
}
#endif

#endif
