/*  
    Copyright (C) 2014 Andreas Stewering-Bone

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. */

/**
 * @file cs_calibration_utils.h
 * @author Andreas Stewering-Bone
 * @date 9 Oct 2014
 * @brief File containing utils for calibration protocols.
 *
 * File containing utils for calibration protocols.
*/
#ifndef CS_CALIBRATION_UTILS_H
#define CS_CALIBRATION_UTILS_H

#include <inttypes.h>
#include <math.h>

#include "cs_a2l.h"

typedef union{
    uint8_t u8[2];
    uint16_t u16;
} cs_cu_data16_t;

typedef union{
    uint8_t u8[4];
    uint16_t u16[2];
    uint32_t u32;
    float flt;
} cs_cu_data32_t;

typedef union{
    uint8_t u8[8];
    uint16_t u16[4];
    uint32_t u32[2];
    uint64_t u64;
    float flt[2];
    double dbl;
} cs_cu_data64_t;


#ifdef __cplusplus
extern "C" {
#endif


  /** Function which calc a double period based on ts unit and ts period
      \retval Period in s
  */
  double cs_cu_ts_unit2double(
			       cs_a2l_ts_unit_t ts_unit,     /**< Unit of timestamp*/
			       uint8_t period_unit           /**< Number of units in one period*/
			      );


  /** Convert timestamp into a timespec structure, look for timestamp resolution
      \retval Result timespec structure
  */
  struct timespec cs_cu_ts_timestamp2ts(
					uint32_t timestamp,         /**< Timestamp to convert*/
					cs_a2l_ts_unit_t resolution /**< Unit of timestamp*/
					);


  /** Function detect endianess of local CPU
      \retval 0 Intel
      \retval 1 Motorola
  */
  cs_a2l_bo_t cs_cu_endianess();

  /** Function swap raw data to processor specific data, type uint16_t
      \retval swapped uint16_t
  */
  uint16_t cs_cu_swap16(
			cs_a2l_bo_t endianess,  /**< Remote ECU Byteorder 1 => Motorolla, 0 => Intel*/
			uint16_t value      /**< Value to convert*/
			);

  /** Function swap raw data to processor specific data, type uint32_t
      \retval swapped uint32_t
  */
  uint32_t cs_cu_swap32(
			cs_a2l_bo_t endianess,    /**< Remote ECU Byteorder 1 => Motorolla, 0 => Intel*/
			uint32_t value        /**< Value to convert*/
			);

  /** Function swap raw data to processor specific data, type uint64_t
      \retval swapped uint32_t
  */
  uint64_t cs_cu_swap64(
			cs_a2l_bo_t endianess,    /**< Remote ECU Byteorder 1 => Motorolla, 0 => Intel*/
			uint64_t value            /**< Value to convert*/
			);
    
#ifdef __cplusplus
    }
#endif

#endif
