/*  Copyright (C) 2013 Andreas Stewering-Bone

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. */
#ifndef CS_DEBUG_PRIVATE_H
#define CS_DEBUG_PRIVATE_H

#include <inttypes.h>
#include <stdarg.h>
#include "cs_debug.h"

#define _int_error(...) (cs_error(CS_DEBUG_INT,__func__,__LINE__,__VA_ARGS__))
#define _int_warn(...) (cs_warn(CS_DEBUG_INT,__func__,__LINE__,__VA_ARGS__))
#define _int_info(...) (cs_info(CS_DEBUG_INT,__func__,__LINE__,__VA_ARGS__))
#define _int_debug(...) (cs_debug(CS_DEBUG_INT,__func__,__LINE__,__VA_ARGS__))
#define _int_verbose(...) (cs_verbose(CS_DEBUG_INT,__func__,__LINE__,__VA_ARGS__))
#define _int_error_buffer(DATA,LEN) (cs_error_buffer(CS_DEBUG_INT,__func__,__LINE__,DATA,LEN))
#define _int_warn_buffer(DATA,LEN) (cs_warn_buffer(CS_DEBUG_INT,__func__,__LINE__,DATA,LEN))
#define _int_info_buffer(DATA,LEN) (cs_info_buffer(CS_DEBUG_INT,__func__,__LINE__,DATA,LEN))
#define _int_debug_buffer(DATA,LEN) (cs_debug_buffer(CS_DEBUG_INT,__func__,__LINE__,DATA,LEN))
#define _int_verbose_buffer(DATA,LEN) (cs_verbose_buffer(CS_DEBUG_INT,__func__,__LINE__,DATA,LEN))
#define _int_verbose_binbuffer(DATA,LEN) (cs_verbose_binbuffer(CS_DEBUG_INT,__func__,__LINE__,DATA,LEN))

#ifdef __cplusplus
extern "C" {
#endif
  /** Helper function to generate a variable length char buffer, based on fmt and ap
      \retval char buffer
  */
  char *cs_debug_makemessage(const char *fmt, va_list ap);
  
  /** Set debug level
      \retval Active Debuglevel
  */  
  cs_debug_level_t cs_debuglevel(
				 cs_debug_sub_t sub,     /**< Target subsystem*/
				 cs_debug_level_t level  /**< Debug level*/
				 );

  /** Set debugging callback routine
      Default: Function registered which prints debugging to STDOUT
  */
  void cs_debug_setcallback(
			    cs_debug_sub_t sub,                    /**< Subsystem ID*/
			    void *ctx,                             /**< Callback private data*/
			    void (*callb)(void *ctx,
					  cs_debug_sub_t sub,
					  cs_debug_level_t dl,
					  const char *func,
					  const int line,
					  const char*fmt,
					  va_list ap)              /**< Callback routine*/
			    );

  void cs_error(cs_debug_sub_t sub,
		const char *func,
		const int line,
		const char *fmt, ...);

  void cs_warn(cs_debug_sub_t sub,
	       const char *func,
	       const int line,
	       const char *fmt, ...);
  
  void cs_info(cs_debug_sub_t sub,
	       const char *func,
	       const int line,
	       const char *fmt, ...);

  void cs_debug(cs_debug_sub_t sub,
		const char *func,
		const int line,
		const char *fmt, ...);

  void cs_verbose(cs_debug_sub_t sub,
		  const char *func,
		  const int line,
		  const char *fmt, ...);

  void cs_info_buffer(
		      cs_debug_sub_t sub,
		      const char *func,
		      const int line,
		      uint8_t *data,
		      int len
		      );

  void cs_info_char_buffer(
			   cs_debug_sub_t sub,
			   const char *func,
			   const int line,
			   uint8_t *data,
			   int len
			   );

  void cs_error_buffer(
		       cs_debug_sub_t sub,
		       const char *func,
		       const int line,
		       uint8_t *data,
		       int len
		       );

  void cs_warn_buffer(
		      cs_debug_sub_t sub,
		      const char *func,
		      const int line,
		      uint8_t *data,
		      int len
		      );
  
  void cs_debug_buffer(
		       cs_debug_sub_t sub,
		       const char *func,
		       const int line,
		       uint8_t *data,
		       int len
		       );
  
  void cs_verbose_buffer(
			 cs_debug_sub_t sub,
			 const char *func,
			 const int line,
			 uint8_t *data,
			 int len
			 );

    void cs_verbose_binbuffer(
			      cs_debug_sub_t sub,
			      const char *func,
			      const int line,
			   uint8_t *data,
			      int len
			      );
  
#ifdef __cplusplus
}
#endif

#endif
