/*  Offline file data layer
    Copyright (C) 2015 Andreas Stewering-Bone

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "config.h"
#include "cs_defines.h"
#include "cs_file_parser.h"
#include "cs_file_parser_private.h"
#include "cs_asc_parser.h"
#include "cs_trc_parser.h"



/**************************************************************************/
cs_file_parser_t * cs_file_parser_open(
				       char* filename,
				       cs_file_parser_type_t ftype
				       )
/**************************************************************************/
{
  cs_file_parser_t *rv = NULL;
  
  if((filename == NULL))
    return NULL;


  if(ftype == FILE_PARSER_TYPE_UNKNOWN){
    return NULL;
  }

  rv = (cs_file_parser_t *)malloc(sizeof(cs_file_parser_t));

  if(rv == NULL)
    return NULL;
  
  memset(rv,0,sizeof(cs_file_parser_t));
  rv->ftype = ftype;

  switch(rv->ftype){
  case FILE_PARSER_TYPE_ASC:
    if(cs_asc_parser_open(rv,filename)){
      free(rv);
      return NULL;
    }
    return rv;
  case FILE_PARSER_TYPE_TRC:
    if(cs_trc_parser_open(rv,filename)){
      free(rv);
      return NULL;
    }
    return rv;
  default:
    free(rv);
    return NULL;
  }
  
  return rv;
}

/**************************************************************************/
int cs_file_parser_getentry(
			    cs_file_parser_t *data,
			    cs_file_parser_entry_t *entry
			    )
/**************************************************************************/
{

  if(data == NULL)
    return CS_FAIL;

  if(entry == NULL)
    return CS_FAIL;

  switch(data->ftype){
  case FILE_PARSER_TYPE_ASC:
    return cs_asc_parser_getentry(data,entry);
  case FILE_PARSER_TYPE_TRC:
    return cs_trc_parser_getentry(data,entry);
  default:
    return CS_FAIL;
  }
  return CS_OK;
}

/**************************************************************************/
int cs_file_parser_close(
			 cs_file_parser_t *data
			 )
/**************************************************************************/
{
  if(data == NULL)
    return CS_FAIL;

  switch(data->ftype){
  case FILE_PARSER_TYPE_ASC:
    cs_asc_parser_close(data);
    break;
  case FILE_PARSER_TYPE_TRC:
    cs_trc_parser_close(data);
    break;
  case FILE_PARSER_TYPE_UNKNOWN:
  case FILE_PARSER_TYPE_LOG:
    //Nothing todo
    break;
  }
  
  free(data);
  data = NULL;
  return CS_OK;
}
