/*  
    Copyright (C) 2018 Andreas Stewering-Bone

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. */

/**
 * @file cs_lin_bit_private.h
 * @author Andreas Stewering-Bone
 * @date 9 March 2018
 * @brief File containing lin bus bit interface private header.
 *
 * File containing lin bus bit interface private header.
*/
#ifndef CS_LIN_BIT_PRIVATE_H
#define CS_LIN_BIT_PRIVATE_H

#include <inttypes.h>

#include "cs_lin_bit.h"

/** \enum cs_lin_phys_bit_state_t
    Bit field scan status
*/
typedef enum{
  B_STATE_IDLE = 0,                       /**< No frame*/
  B_STATE_BREAK,                          /**< Break field (low part)*/
  B_STATE_SYNC,                           /**< Sync field*/
  B_STATE_DATA,                           /**< Data or CRC field*/
}cs_lin_phys_bit_state_t;

/** \def CS_LIN_PHYS_BIT_DATA_STORE
    Storage space for internal data store
    //8 Byte data + PID + CRC + SYNC
*/
#define CS_LIN_PHYS_BIT_DATA_STORE (CS_LIN_MAXDLC + 3)

/** \enum cs_lin_phys_bit_data_state_t
    Bit field data scan status
*/
typedef enum{
  B_STATE_DATA_IDLE = 0,                  /**< Nothing*/
  B_STATE_DATA_START,                     /**< Start bit*/
  B_STATE_DATA_DATA,                      /**< Inside Data*/
}cs_lin_phys_bit_data_state_t; 

/** \struct cs_lin_phys_bit
    Storage structure for transfer LIN via digital out
 */
struct cs_lin_phys_bit{
  double lin_bits;       /**< Interface bits per LIN Bit*/
  uint8_t *if_break;     /**< Precalulated break field*/
  int if_break_len;      /**< Precalculated break field length in byte*/
  uint8_t *if_sync;      /**< Precalulated sync field*/
  int if_sync_len;       /**< Precalculated sync field length in byte*/
  double if_byte_shift;  /**< Byte shift*/
  double if_byte_shift_header;  /**< Byte shift, header part*/
  int min_size;          /**< Minimum interface buffer size in byte for a maximum length LIN Frame*/
  cs_lin_phys_bit_state_t scan_state; /**< General scan state*/
  cs_lin_phys_bit_data_state_t scan_data_state; /**< Data scan state*/
  int counter;           /**< Bit counter*/
  int data_idle_counter; /**< Data idle counter*/
  int data_bit_index;    /**< Bit index of actual byte*/
  int data_byte_index;   /**< Byte index of actual frame*/
  uint8_t frame_data[CS_LIN_PHYS_BIT_DATA_STORE]; /**< Temp data storage, max data length + CRC*/
  double data_start_offset; /**< Measured middle of a bit*/
  double data_sample_int;   /**< Sample point integrator*/
  double minbreak;      /**< Minimal break size*/
  double maxbreak;      /**< Maximal break size*/
  double minbitlength;  /**< Minimal bit length*/
  double maxbitlength;  /**< Maximal bit length*/
  uint64_t buffercounter; /**< Buffer call counter*/
};

#ifdef __cplusplus
extern "C" {
#endif



#ifdef __cplusplus
}
#endif

#endif
