/*  
    Copyright (C) 2017 Andreas Stewering-Bone

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. */

/**
 * @file cs_lin_sllin.h
 * @author Andreas Stewering-Bone
 * @date 9 Oct 2017
 * @brief File containing lin bus linux sllin specific header.
 *
 * File containing lin bus linux sllin specific header.
*/
#ifndef CS_LIN_SLLIN_H
#define CS_LIN_SSLIN_H

#include <inttypes.h>

#include "cs_socketcan.h"


#ifdef __cplusplus
extern "C" {
#endif

  /** Configure SLLin frame cache, basic configuration and slave data update
      \retval 0 No failure
      \retval 1 Failure
   */
  int cs_lin_sllin_configure_cache(
				   cs_cocketcan_t *sk,   /**< Socketcan structure */
				   uint32_t id,          /**< Lin Id*/
				   int dlc,              /**< Data DLC*/
				   uint8_t *data         /**< Data buffer*/
				   );

  /** SLLin master send data
      \retval 0 No failure
      \retval 1 Failure
   */
  int cs_lin_sllin_master_send(
			       cs_cocketcan_t *sk,   /**< Socketcan structure */
			       uint32_t id,          /**< Lin Id*/
			       int dlc,              /**< Data DLC*/
			       uint8_t *data         /**< Data buffer*/
			       );

  /** SLLin master send data request
      \retval 0 No failure
      \retval 1 Failure
  */
  int cs_lin_sllin_master_request(
				  cs_cocketcan_t *sk,   /**< Socketcan structure */
				  uint32_t id           /**< Lin Id*/
				  );

    /** SSLIN receive non blocking
      \retval CS_LIN_SLLIN_OK No failure
      \retval CS_LIN_SLLIN_FAIL General Failure
      \retval CS_LIN_SLLIN_ERROR Receive Error frame
      \retval CS_LIN_SLLIN_RTR Receive RTR frame
      \retval CS_LIN_SLLIN_TIMEOUT Receive timeout
  */
  int cs_lin_sllin_recv_nonblock(
				 cs_socketcan_t *sk,  /**< Socket struct pointer*/
				 uint32_t *id,        /**< CAN id storage pointer*/
				 uint8_t *buffer,     /**< Receive data buffer*/
				 int *len,            /**< Receive buffer length, should be initialized with min 8 Bytes*/
				 struct timeval *tv   /**< Receive timestamp pointer*/
				 );
  


  
  
#ifdef __cplusplus
}
#endif

#endif
