/* 
    Copyright (C) 2018 Andreas Stewering-Bone

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. */
/**
 * @file cs_message_private.h
 * @author Andreas Stewering-Bone
 * @date 20 Dec 2018
 * @brief File containing restbus simulation message private header.
 *
 * File containing restbus simulation message handling private header.
 */
#ifndef CS_CAN_MESSAGE_PRIVATE_H
#define CS_CAN_MESSAGE_PRIVATE_H

#include <inttypes.h>
#include <stdarg.h>
#include "cs_defines.h"
#include "cs_lists.h"
#include "cs_message.h"

/** \struct cs_message_t
    CAN message abstraction
*/
struct cs_message{
  uint32_t id;                         /**< CAN id*/
  uint8_t dlc;                         /**< CAN DLC*/
  char name[CS_CHARBUF];               /**< Message name*/
  char producer[CS_CHARBUF];           /**< Message producer*/
  cs_message_dir_t direction;          /**< Message direction*/
  cs_message_type_t mtype;             /**< Message type*/
  cs_message_send_type_t send_type;    /**< Send type*/
  cs_message_ptype_t ptype;            /**< Physical layer format, CAN-2.0 vs CAN-FD*/
  uint32_t interval;                   /**< Send interval in 0.000001s*/
  uint32_t interval_offset;            /**< Offset to base cycle in 0.000001s*/
  uint32_t interval_startoffset;       /**< Offset at start in 0.000001s*/
  cs_lists_t listsignal;               /**< Signal list*/
  struct timeval ts;                   /**< Receive timestamp*/
  struct timeval *externts;            /**< Pointer to external registered receive timestamp*/
  double *externtvh;                   /**< Pointer to external registered receive ts in s*/
  uint8_t *externbuffer;               /**< Pointer to external registered send/receive rawbuffer*/
  uint32_t timeout;                    /**< Receive timeout in 0.000001s*/
  uint8_t *timeoutflag;                /**< Extern registered timeout flag*/
  uint64_t msg_counter;                /**< Message send/recv counter, for future use*/
  uint8_t default_rawvalue[CS_DATA];   /**< Default raw value*/
  uint8_t *rawbuffer;                  /**< External raw value buffer*/
};

#endif
