/*  
    Copyright (C) 2019 Andreas Stewering-Bone

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. */
/**
 * @file cs_schedule_private.h
 * @author Andreas Stewering-Bone
 * @date 3 Jan 2019
 * @brief File containing restbus simulation message scheduling private definitions.
 *
 * File containing restbus simulation message scheduling private definitions.
 */
#ifndef CS_SCHEDULE_PRIVATE_H
#define CS_SCHEDULE_PRIVATE_H

#include <inttypes.h>
#include <cs_schedule.h>
#include <cs_message.h>
#include <cs_lists.h>
#include <cs_socketcan.h>

/** \struct cs_schedule_message_t
    Internal used structure to manage messages
*/ 
typedef struct{
  uint32_t counter;               /**< Time interval counter, resolution 0.000001s*/
  uint32_t cycle;                 /**< Message send/recv cycle, resolution 0.000001s*/
  uint32_t timeout;               /**< Message recv timeout, resolution 0.000001s*/
  uint32_t id;                    /**< Copy of CAN id*/
  cs_schedule_msgstate_t state;   /**< Message state*/
  cs_schedule_msgaction_t action; /**< Message action*/
  cs_socketcan_send_mode_t smode; /**< Send mode, CAN-2.0 vs CAN-FD*/
  cs_message_t *message;          /**< Message content structure*/
} cs_schedule_message_t;

/** \struct cs_schedule
    Main scheduler work structure
*/
struct cs_schedule{
  cs_socketcan_t *socket;        /**< CAN interface socket structure*/
  uint32_t cycle;                /**< Call cycle, resolution 0.000001s*/
  cs_lists_t l_send;             /**< Message list send*/
  cs_lists_t l_recv;             /**< Message list recv*/
  int max_messpcycle;            /**< Maximum number of send message send per one scheduler call*/
  uint32_t send_no_messages;     /**< Number of messages to send in actual cycle*/
  uint32_t recv_no_messages;     /**< Number of messages recieved in actual cycle*/
  uint32_t send_shifted_messages;/**< Number of shifted messages to the next cycle,
				    caused by limitation of shifted_messages*/
  uint32_t send_tx_full_counter; /**< Failure counter for TX-Queque-FULL Error on pyhsical layer*/
  uint32_t send_error_counter;   /**< General Failure counter for send packages on pyhsical layer*/
};
#endif
