/*  
    Copyright (C) 2017 Andreas Stewering-Bone

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. */

/**
 * @file cs_time.h
 * @author Andreas Stewering-Bone
 * @date 10 Mar 2017
 * @brief File containing time utils.
 *
 * File containing time utils.
*/
#ifndef CS_TIME_H
#define CS_TIME_H
#include <time.h>
#include <inttypes.h>

#ifdef __cplusplus
extern "C" {
#endif
  /** Convert timeout (ms) into a timespec structure
      \retval Result timespec structure
  */
  struct timespec cs_ts_timeout2ts(
				   uint32_t timeout
				   );

  /** Build sum of 2 timespec structures
      \retval Result timespec structure
  */
  struct timespec cs_ts_addtimespec(
				    struct timespec time1, 
				    struct timespec time2
				    );

  /** Build diff of 2 timespec structures
      \retval Result timespec structure
  */
  struct timespec cs_ts_difftimespec(
				     struct timespec start,
				     struct timespec end
				     );
  
  /** Add timeout value to timespec structure, timeout in ms
   */
  void cs_ts_add_timeout(
			 struct timespec *ts, 
			 uint32_t timeout
			 );

  /** Calc future time base on now plus timeout in ms
   */
  void cs_ts_next(
		  struct timespec *ts,
		  uint32_t timeout
		  );
  
  /** Compare two timestamps
      \retval 0 if ta < tb
      \retval 1 if ta >= tb
  */
  int cs_ts_cmp(
		struct timespec *ta,
		struct timespec *tb
		);

  /** Get actual time
   */
  void cs_ts_gettime(
		     struct timespec *ts
		     );

  /** get actual time
   */
  void cs_ts_gettimeval(
			struct timeval *tv
			);
  
  /** Convert timespec to timeval value
      \retval time is struct timeval
  */
  struct timeval cs_ts_timespec2timeval(
					struct timespec time
					);  

  /** Convert timespec to double value
      \retval time is second
  */
  double cs_ts_timespec2d(
			  struct timespec time
			  );

  /** Convert timespec to char buffer
  */
  void cs_ts_timespec2c(struct timespec time,
			 char *buffer,
			 int len
			 );
  
  /** Convert timespec to double value
      \retval time is second
  */
  double cs_ts_timeval2d(
			 struct timeval time
			 );

  /** Check if timestamp is in the past
      \retval 0 if timestamp is in the future
      \retval 1 if timestamp is in the past
  */
  int cs_ts_checkpast(
		      struct timespec *ts
		      );
  /** Assemble timval from sec and usec value
      \retval time in struct timeval
  */
  struct timeval cs_ts_assembletimeval(
				       int sec,       /**< Seconds*/
				       int usec       /**< USeconds*/
				       );
  
  /** Function timeout relative sleep for timeout ms
   */
  void cs_ts_sleep(
		   uint32_t timeout               /**< Time in 0.001s*/
		   );

  /** Sleep until absolute timestamp
   */ 
  void cs_ts_sleep_next(
			struct timespec *ts
			);
  
  
#ifdef __cplusplus
    }
#endif
  
#endif
