/*  CCP CAN Stack for SocketCAN
    Copyright (C) 2015 Andreas Stewering-Bone

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "config.h"
#include "cs_defines.h"
#include "cs_parser.h"
#include "cs_debug_private.h"
#include "cs_trc_parser.h"

/**************************************************************************/
int cs_trc_parser_open(
			cs_file_parser_t *data,
			char* filename
			)
/**************************************************************************/
{
  char regex[CS_REGEX_BUFFER*2];

  if((data == NULL)||(filename == NULL)){
    _int_error("Parameter failure\n");
    return CS_FAIL;
  }

  data->line = 0;
  data->len = 0;
  data->f = NULL;

  data->f = fopen(filename, "r");
  if(data->f == NULL){
    _int_error("Failed to open file\n");
    return CS_FAIL;
  }

  //Build up regex
  memset(regex,0,sizeof(char)*CS_REGEX_BUFFER*2);
  //     2)         5.0  Rx         0137  8  00 E4 FF AA 00 FC C2 07 
snprintf(regex,CS_REGEX_BUFFER*2,"(?'index'[0-9]+)\\)[\\s]+(?'ts'[0-9.]+)[\\s]+(?'dir'Rx|Tx)[\\s]+(?'id'[0-9A-Fa-f]+)[\\s]+(?'dlc'[0-9]+)[\\s]+(?'v0'[0-9A-F]{2})?[\\s]?(?'v1'[0-9A-F]{2})?[\\s]?(?'v2'[0-9A-F]{2})?[\\s]?(?'v3'[0-9A-F]{2})?[\\s]?(?'v4'[0-9A-F]{2})?[\\s]?(?'v5'[0-9A-F]{2})?[\\s]?(?'v6'[0-9A-F]{2})?[\\s]?(?'v7'[0-9A-F]{2})?");  


 //Precompile regex
  if((data->re = (void *)cs_parser_precompile((const char*)regex)) == NULL){
    _int_error("Regex precompilation failed\n");
    return CS_FAIL;
  }

  return CS_OK;
}

/**************************************************************************/
int cs_trc_parser_getentry(
			     cs_file_parser_t *data,
			     cs_file_parser_entry_t *entry
			     )
/**************************************************************************/
{
  int read;
  int comment = 1;
  int rv = 0;
  int noresults = 20;
  int ovector[CS_REGEX_RESULTSIZE*noresults];
  int canid;
  int i;
  char label[CS_CHARBUF];
  int ldata;

  if((data == NULL) ||
     (entry == NULL)){
    _int_error("Parameter failure\n");
    return CS_FAIL;
  }

  pcre *re = (pcre *)data->re;

  memset(entry,0,sizeof(cs_file_parser_entry_t));

  while(comment){
    read = getline(&data->line, &data->len, data->f);
    
    if(read <= 0){
      return CS_FAIL;
    }
    if(data->len == 0){
      return CS_FAIL;
    }
    //_int_verbose("Line: %s",data->line);
    if(memcmp(&data->line[0],";",1)==0){
      comment = 1;
    } else {
      comment = 0;
    }
  }
  
  rv = pcre_exec(re, NULL, (const char*)data->line,data->len, 0,PCRE_NOTEMPTY, ovector, CS_REGEX_RESULTSIZE*noresults);

  if(rv < 0) {
    switch(rv) {
    case PCRE_ERROR_NOMATCH:
      _int_verbose("No match\n");
      return CS_FAIL;
      break;
    default: 
      _int_error("Matching error %d\n", rv);
      return CS_FAIL;
      break;
    }
  } else {
    //Get index
    if(/*cs_parser_getsubstring_int(re,data->line, ovector, CS_REGEX_RESULTSIZE*noresults,"index",
	 &entry->id,0) ||*/
       cs_parser_getsubstring_double(re,data->line, ovector, CS_REGEX_RESULTSIZE*noresults,"ts",
				     &entry->ts,0.0)||
       cs_parser_getsubstring_hex(re,data->line, ovector, CS_REGEX_RESULTSIZE*noresults,"id",
				  &canid,0) ||
       cs_parser_getsubstring_int(re,data->line, ovector, CS_REGEX_RESULTSIZE*noresults,"dlc",
				  &entry->dlc,0)
       ){
      _int_error("Failed to find substrings index, ts, id, dlc\n");
      return CS_FAIL;
    } else {
      //From ms to sec
      entry->ts = entry->ts*1e-3;
      entry->canid = (uint32_t)canid;
    }
    if(entry->dlc > FILE_PARSER_DLC_MAX){
      _int_error("Invalid data length\n");
      return CS_FAIL;
    }
    for(i = 0; i< entry->dlc;i++){
      memset(label,0,sizeof(char)*CS_CHARBUF);
      snprintf(label,CS_CHARBUF,"v%i",i);
      if(cs_parser_getsubstring_hex(re,data->line, ovector, CS_REGEX_RESULTSIZE*noresults,label,
				    &ldata,0)){
	_int_error("Failed to find substring %s\n",label);
	return CS_FAIL;
      } else {
	entry->data[i] = (uint8_t)ldata;
      }
    }
  }

  return CS_OK;
}

/**************************************************************************/
int cs_trc_parser_close(
			cs_file_parser_t *data
			)
/**************************************************************************/
{
  if(data == NULL){
    _int_error("Parameter failure\n");
    return CS_FAIL;
  }

  if(data->re != NULL)
    pcre_free((pcre *)data->re);

  if(data->line != NULL)
    free(data->line);

  data->line = NULL;

  if(data->f != NULL)
    fclose(data->f);

  data->f = NULL;

  return CS_OK;
}
