/*  uds.h --  UDS (ISO 14229-1) Stack
    Copyright (C) 2013 Andreas Stewering-Bone

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. */

/**
 * @file cs_uds_private.h
 * @author Andreas Stewering-Bone
 * @date 9 Oct 2014
 * @brief File containing the UDS specific private structures.
 *
 * This file contains the UDS specific private structure definitions.
*/

#ifndef CS_UDS_PRIVATE_H
#define CS_UDS_PRIVATE_H

#include <inttypes.h>

#include "cs_defines.h"
#include "cs_uds.h"
#include "cs_isotp.h"
#include "cs_time.h"

/** \enum a_tat_t
    Application layer target address type
*/
typedef enum{
  PHYS=0,      /**< Physical addressing*/
  FUNC         /**< Functional addressing*/
}a_tat_t;

/** \enum a_mt_t
    Application layer message type
*/
typedef enum{
  DIAG=0,      /**< Diagnostic*/
  RDIAG        /**< Remote diagnostic*/
}a_mt_t;

/**< \enum a_pci_t
   Protocol control information
*/
typedef struct{
  uint8_t nrsi;  /**< negative response*/
  uint8_t si;    /**< Service identifier*/
} a_pci_t;


/** \struct cs_uds_data_t
    UDS paket definition
*/
typedef struct{
  a_mt_t mt;                         /**< Application layer message type*/
  uint32_t sa;                       /**< Source CAN id*/
  uint32_t ta;                       /**< Target CAN id*/
  a_tat_t tat;                       /**< Application layer target address type*/
  uint16_t ae;                       /**< Address extender*/
  uint32_t ra;                       /**< Remote address*/
  a_pci_t pci;                       /**< Protocol control information*/
  int result;                        /**< Internal access storage, used by server stack*/
  resp_t resp;                       /**< Is response requested*/
  uint32_t lt;                       /**< Data segement length*/
  uint8_t a_data[CS_UDS_MAXBYTE];    /**< Data segment*/
  struct timeval tv_response;        /**< Response timestamp*/
} cs_uds_data_t;

/** \struct cs_uds_service_t
    Basic UDS stack storage structure
*/
struct cs_uds_service{
  char ifcan[CS_CHARBUF];             /**< Interface name*/
  cs_isotp_t *isotp;                  /**< IsoTP data*/
  uint32_t sa;                        /**< Source CAN id*/
  uint32_t ta;                        /**< Target CAN id*/
  struct cs_uds_time_server tserver;  /**< Stack timing values, server values*/
  struct cs_uds_time_client tclient;  /**< Stack timing values, client values*/
  struct timespec ts_nextrequest;     /**< Timestamp for next client request*/
  uint16_t lerror;                    /**< Last error flag*/
  void *srv_ctx;                      /**< Server private data pointer*/
  cs_uds_server_cb_t srv_cb;          /**< Server callback function table*/
};

#endif
