/*  XCP CAN Stack for SocketCAN
    Copyright (C) 2014 Andreas Stewering-Bone

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/mman.h>
#include <unistd.h>

#include "cs_xcp_a2l.h"
#include "cs_defines.h"
#include "cs_calibration_utils.h"
#include "cs_xcp_private.h"

/**************************************************************************/
int _cs_xcp_a2l_GetOptional(
			    cs_xcp_a2l_t *data,
			    cs_xcp_a2l_commode_t *mode
			    )
/**************************************************************************/
{
  pcre *re = NULL;
  char regex[CS_REGEX_BUFFER];
  int rv = 0;
  int noresult = 2;
  int ovector[CS_REGEX_RESULTSIZE*noresult];
  char namedvalue[CS_PARSER_CHARBUF];
  struct optlist{
    char option[255];
    uint8_t id;
  };
  int i;
  int found;
  const struct optlist list[]={
			       {"OPTIONAL_CMD GET_COMM_MODE_INFO",XCP_CMD_GET_COMM_MODE_INFO},
			       {"OPTIONAL_CMD GET_ID",XCP_CMD_GET_ID},
			       {"OPTIONAL_CMD SET_REQUEST",XCP_CMD_SET_REQUEST},
			       {"OPTIONAL_CMD GET_SEED",XCP_CMD_GET_SEED},
			       {"OPTIONAL_CMD UNLOCK",XCP_CMD_UNLOCK},
			       {"OPTIONAL_CMD SET_MTA",XCP_CMD_SET_MTA},
			       {"OPTIONAL_CMD UPLOAD",XCP_CMD_UPLOAD},
			       {"OPTIONAL_CMD SHORT_UPLOAD",XCP_CMD_SHORT_UPLOAD},
			       {"OPTIONAL_CMD BUILD_CHECKSUM",XCP_CMD_BUILD_CHECKSUM},
			       {"OPTIONAL_CMD TRANSPORT_LAYER_CMD",XCP_CMD_TRANSPORT_LAYER_CMD},
			       {"OPTIONAL_CMD USER_CMD",XCP_CMD_USER_CMD},
			       {"OPTIONAL_CMD DOWNLOAD",XCP_CMD_DOWNLOAD},
			       {"OPTIONAL_CMD DOWNLOAD_NEXT",XCP_CMD_DOWNLOAD_NEXT},
			       {"OPTIONAL_CMD DOWNLOAD_MAX",XCP_CMD_DOWNLOAD_MAX},
			       {"OPTIONAL_CMD SHORT_DOWNLOAD",XCP_CMD_SHORT_DOWNLOAD},
			       {"OPTIONAL_CMD MODIFY_BITS",XCP_CMD_MODIFY_BITS},
			       {"OPTIONAL_CMD SET_CAL_PAGE",XCP_CMD_SET_CAL_PAGE},
			       {"OPTIONAL_CMD GET_CAL_PAGE",XCP_CMD_GET_CAL_PAGE},
			       {"OPTIONAL_CMD GET_PAG_PROCESSOR_INFO",XCP_CMD_GET_PAG_PROCESSOR_INFO},
			       {"OPTIONAL_CMD GET_SEGMENT_INFO",XCP_CMD_GET_SEGMENT_INFO},
			       {"OPTIONAL_CMD GET_PAGE_INFO",XCP_CMD_GET_PAGE_INFO},
			       {"OPTIONAL_CMD SET_SEGMENT_MODE",XCP_CMD_SET_SEGMENT_MODE},
			       {"OPTIONAL_CMD GET_SEGMENT_MODE",XCP_CMD_GET_SEGMENT_MODE},
			       {"OPTIONAL_CMD COPY_CAL_PAGE",XCP_CMD_COPY_CAL_PAGE},
			       {"OPTIONAL_CMD CLEAR_DAQ_LIST",XCP_CMD_CLEAR_DAQ_LIST},
			       {"OPTIONAL_CMD SET_DAQ_PTR",XCP_CMD_SET_DAQ_PTR},
			       {"OPTIONAL_CMD WRITE_DAQ",XCP_CMD_WRITE_DAQ},
			       {"OPTIONAL_CMD SET_DAQ_LIST_MODE",XCP_CMD_SET_DAQ_LIST_MODE},
			       {"OPTIONAL_CMD GET_DAQ_LIST_MODE",XCP_CMD_GET_DAQ_LIST_MODE},
			       {"OPTIONAL_CMD START_STOP_DAQ_LIST",XCP_CMD_START_STOP_DAQ_LIST},
			       {"OPTIONAL_CMD START_STOP_SYNCH",XCP_CMD_START_STOP_SYNCH},
			       {"OPTIONAL_CMD GET_DAQ_CLOCK",XCP_CMD_GET_DAQ_CLOCK},
			       {"OPTIONAL_CMD READ_DAQ",XCP_CMD_READ_DAQ},
			       {"OPTIONAL_CMD GET_DAQ_PROCESSOR_INFO",XCP_CMD_GET_DAQ_PROCESSOR_INFO},
			       {"OPTIONAL_CMD GET_DAQ_RESOLUTION_INFO",XCP_CMD_GET_DAQ_RESOLUTION_INFO},
			       {"OPTIONAL_CMD GET_DAQ_LIST_INFO",XCP_CMD_GET_DAQ_LIST_INFO},
			       {"OPTIONAL_CMD GET_DAQ_EVENT_INFO",XCP_CMD_GET_DAQ_EVENT_INFO},
			       {"OPTIONAL_CMD FREE_DAQ",XCP_CMD_FREE_DAQ},
			       {"OPTIONAL_CMD ALLOC_DAQ",XCP_CMD_ALLOC_DAQ},
			       {"OPTIONAL_CMD ALLOC_ODT",XCP_CMD_ALLOC_ODT},
			       {"OPTIONAL_CMD ALLOC_ODT_ENTRY",XCP_CMD_ALLOC_ODT_ENTRY},
			       {"OPTIONAL_CMD PROGRAM_START",XCP_CMD_PROGRAM_START},
			       {"OPTIONAL_CMD PROGRAM_CLEAR",XCP_CMD_PROGRAM_CLEAR},
			       {"OPTIONAL_CMD PROGRAM",XCP_CMD_PROGRAM},
			       {"OPTIONAL_CMD PROGRAM_RESET",XCP_CMD_PROGRAM_RESET},
			       {"OPTIONAL_CMD GET_PGM_PROCESSOR_INFO",XCP_CMD_GET_PGM_PROCESSOR_INFO},
			       {"OPTIONAL_CMD GET_SECTOR_INFO",XCP_CMD_GET_SECTOR_INFO},
			       {"OPTIONAL_CMD PROGRAM_PREPARE",XCP_CMD_PROGRAM_PREPARE},
			       {"OPTIONAL_CMD PROGRAM_FORMAT",XCP_CMD_PROGRAM_FORMAT},
			       {"OPTIONAL_CMD PROGRAM_NEXT",XCP_CMD_PROGRAM_NEXT},
			       {"OPTIONAL_CMD PROGRAM_MAX",XCP_CMD_PROGRAM_MAX},
			       {"OPTIONAL_CMD PROGRAM_VERIFY",XCP_CMD_PROGRAM_VERIFY},
			       {"",0}
  };


  if((data == NULL) ||
     (mode == NULL)){
    _xcp_error("Parameter failure\n");
    return CS_FAIL;
  }
  
  //Build up regex
  memset(regex,0,sizeof(char)*CS_REGEX_BUFFER);
  snprintf(regex,CS_REGEX_BUFFER,"(?'optional'OPTIONAL_CMD [A-Z_]+)");
  
  //Precompile regex
  if((re = cs_parser_precompile((const char *)regex)) == NULL){
    _xcp_error("Regex precompilation failed\n");
    return CS_FAIL;
  }
  
  //Reset ovector[1], because this is the search offset
  ovector[1] = 0;
  
  while(rv >= 0){
    //Get data, only one fit is allowed
    rv = pcre_exec(re, NULL, (const char*)data->a2l.protocol.result,data->a2l.protocol.resultlen, ovector[1],PCRE_NOTEMPTY, ovector, CS_REGEX_RESULTSIZE * noresult);
    
    if(rv < 0) {
      switch(rv) {
      case PCRE_ERROR_NOMATCH:
	break;
      default: 
	_xcp_error("Matching error %d\n", rv);
	pcre_free(re);
	return CS_FAIL;
	break;
      }
    } else {
      if(cs_parser_getsubstring(re,data->a2l.protocol.result,ovector,CS_REGEX_RESULTSIZE*noresult,"optional",namedvalue)){
	_xcp_error("Failed to get substring optional\n");
	pcre_free(re);
	return CS_FAIL;
      }
      i = 0;
      found = 0;
      while(strcmp(list[i].option,"")!=0){
	//Compare string
	if(strcmp(namedvalue,list[i].option) == 0){
	  mode->optfeature[list[i].id] = A2L_FEATURE_ENABLED;
	  _xcp_verbose("Enable Feature: %s\n",list[i].option);
	  found = 1;
	  break;
	}
	i++;
      }
      if(!found){
	_xcp_error("Unknown feature %s\n",namedvalue);
      }
    }
  }
    
  pcre_free(re); 

  return CS_OK;
}

/**************************************************************************/
int _cs_xcp_a2l_GetComModeSupported(
				    cs_xcp_a2l_t *data,
				    cs_xcp_a2l_commode_t *mode
				    )
/**************************************************************************/
{
  pcre *re = NULL;
  char regex[CS_REGEX_BUFFER];
  int rv = 0;
  int noresult = 6;
  int ovector[CS_REGEX_RESULTSIZE*noresult];
  char namedvalue[CS_PARSER_CHARBUF];
  unsigned int ldata;
  
  //COMMUNICATION_MODE_SUPPORTED BLOCK SLAVE MASTER 0x0A 0x02

  if((data == NULL) ||
     (mode == NULL)){
    _xcp_error("Parameter failure\n");
    return CS_FAIL;
  }
  
  //Build up regex
  memset(regex,0,sizeof(char)*CS_REGEX_BUFFER);
  snprintf(regex,CS_REGEX_BUFFER,"COMMUNICATION_MODE_SUPPORTED BLOCK(?'slave' SLAVE)?(?'master' MASTER)?(?'stim' 0x[0-9a-fA-F]+)+(?'bs' 0x[0-9a-fA-F]+)+");
  
  //Precompile regex
  if((re = cs_parser_precompile((const char *)regex)) == NULL){
    _xcp_error("Regex precompilation failed\n");
    return CS_FAIL;
  }
  
  //Get data, only one fit is allowed
  rv = pcre_exec(re, NULL, (const char*)data->a2l.protocol.result,data->a2l.protocol.resultlen, 0,PCRE_NOTEMPTY, ovector, CS_REGEX_RESULTSIZE * noresult);
    
  if(rv < 0) {
    switch(rv) {
    case PCRE_ERROR_NOMATCH:
      //Keep mode settings deactivated, because blockmode is not available
      break;
    default: 
      _xcp_error("Matching error %d\n", rv);
      pcre_free(re);
      return CS_FAIL;
      break;
    }
  } else {
    if(!cs_parser_getsubstring(re,data->a2l.protocol.result,ovector,CS_REGEX_RESULTSIZE*noresult,"slave",namedvalue)){
      //Slave block mode is available
      mode->SBM = 1;
    }

    if(!cs_parser_getsubstring(re,data->a2l.protocol.result,ovector,CS_REGEX_RESULTSIZE*noresult,"master",namedvalue)){
      //Master block mode is available
      mode->MBM = 1;
    }

    if(!cs_parser_getsubstring(re,data->a2l.protocol.result,ovector,CS_REGEX_RESULTSIZE*noresult,"bs",namedvalue)){
      if(!sscanf(namedvalue,"%X",&ldata)){
	_xcp_error("Failed to parse substring Block size: %s\n",namedvalue);
	pcre_free(re);
	return CS_FAIL;
      } else {
	mode->max_bs = (uint8_t)ldata;
      }
    } else {
      if(mode->SBM || mode->MBM){
	_xcp_error("Block mode is enabled but no Block size parameter\n");
	pcre_free(re);
	return CS_FAIL;
      }
    }
    
    if(!cs_parser_getsubstring(re,data->a2l.protocol.result,ovector,CS_REGEX_RESULTSIZE*noresult,"stim",namedvalue)){
      if(!sscanf(namedvalue,"%X",&ldata)){
	_xcp_error("Failed to parse substring Block STIM: %s\n",namedvalue);
	pcre_free(re);
	return CS_FAIL;
      } else {
	mode->min_st = (uint8_t)ldata;
      }
    } else { 
      if(mode->SBM || mode->MBM){
	_xcp_error("Block mode is enabled but no STIM parameter\n");
	pcre_free(re);
	return CS_FAIL;
      }
	
    }
    
  }
    
  _xcp_verbose("Master block mode is %s\n",mode->MBM?"available":"not available");
  _xcp_verbose("Slave block mode is %s\n",mode->SBM?"available":"not available");
  _xcp_verbose("Block size:  %i\n",mode->max_bs);
  _xcp_verbose("Block STIM: %i x100nues\n",mode->min_st);

  pcre_free(re); 

  return CS_OK;
}

/**************************************************************************/
int _cs_xcp_a2l_GetSeedKey(
			   cs_xcp_a2l_t *data,
			   cs_xcp_a2l_commode_t *mode
			   )
/**************************************************************************/
{
  pcre *re = NULL;
  char regex[CS_REGEX_BUFFER];
  int rv = 0;
  int noresult = 3;
  int ovector[CS_REGEX_RESULTSIZE*noresult];
  char namedvalue[CS_PARSER_CHARBUF];
  
  if((data == NULL) ||
     (mode == NULL)){
    _xcp_error("Parameter failure\n");
    return CS_FAIL;
  }
  
  //Build up regex
  memset(regex,0,sizeof(char)*CS_REGEX_BUFFER);
  snprintf(regex,CS_REGEX_BUFFER,"SEED_AND_KEY_EXTERNAL_FUNCTION \"(?'algo'.+?)\"");
  
  //Precompile regex
  if((re = cs_parser_precompile((const char *)regex)) == NULL){
    _xcp_error("Regex precompilation failed\n");
    return CS_FAIL;
  }
  
  //Get data, only one fit is allowed
  rv = pcre_exec(re, NULL, (const char*)data->a2l.protocol.result,data->a2l.protocol.resultlen,0,PCRE_NOTEMPTY, ovector, CS_REGEX_RESULTSIZE * noresult);
    
  if(rv < 0) {
    switch(rv) {
    case PCRE_ERROR_NOMATCH:
      //Keep mode settings deactivated
      break;
    default: 
      _xcp_error("Matching error %d\n", rv);
      pcre_free(re);
      return CS_FAIL;
      break;
    }
  } else {
    if(!cs_parser_getsubstring(re,data->a2l.protocol.result,ovector,CS_REGEX_RESULTSIZE*noresult,"algo",namedvalue)){
      strncpy(mode->seedkey,namedvalue,CS_CHARBUF);

      _xcp_verbose("Seed key algorythm: %s\n",mode->seedkey);
    }
  }
    

  pcre_free(re); 

  return CS_OK;
}

/**************************************************************************/
int _cs_xcp_a2l_GetEvents(
			  cs_xcp_a2l_t *data,
			  cs_parser_result_t ldaq, 
			  cs_xcp_a2l_daq_proc_t *daq
			  )
/**************************************************************************/
{
  pcre *re = NULL;
  char regex[CS_REGEX_BUFFER];
  int rv = 0;
  int noresult = 11;
  int ovector[CS_REGEX_RESULTSIZE*noresult];
  char namedvalue[CS_PARSER_CHARBUF];
  unsigned int lvalue;
  unsigned int number;
  int counter = 0;
  cs_parser_result_t levent;
  cs_parser_result_t llocal;

  if((data == NULL) ||
     (daq == NULL)){
    _xcp_error("Parameter failure\n");
    return CS_FAIL;
  }

  memset(daq->event,0,sizeof(cs_xcp_a2l_event_t)*CS_XCP_A2L_EVENT_MAX);

  //Build up regex
  memset(regex,0,sizeof(char)*CS_REGEX_BUFFER);
  snprintf(regex,CS_REGEX_BUFFER,"\"(?'name'.*?)\"+[\\s]+\"(?'shortname'.*?)\"[\\s]+(?'number'0x[0-9a-fA-F]+)[\\s]+(?'type'(DAQ|STIM|DAQ_STIM))[\\s]+(?'maxdaq'0x[0-9a-fA-F]+)[\\s]+(?'cycle'0x[0-9a-fA-F]+)[\\s]+(?'unit'0x[0-9a-fA-F]+)[\\s]+(?'prior'0x[0-9a-fA-F]+)");
  //Precompile regex
  if((re = cs_parser_precompile((const char *)regex)) == NULL){
    _xcp_error("Regex precompilation failed\n");
    return CS_FAIL;
  }
  
  //Define search area
  levent.searchin = ldaq.result;
  levent.searchlen = ldaq.resultlen;

  while(!cs_a2l_search_section(&levent,"EVENT",NULL)){
    if((levent.result != NULL) && (counter < daq->ev_max) && (counter < CS_XCP_A2L_EVENT_MAX)){
      counter++;
      //Get data, only one fit is allowed
      rv = pcre_exec(re, NULL, (const char*)levent.result,levent.resultlen, 0,PCRE_NOTEMPTY, ovector, CS_REGEX_RESULTSIZE * noresult);
      
      if(rv < 0) {
	switch(rv) {
	case PCRE_ERROR_NOMATCH:
	  _xcp_debug("No Event match\n");
	  break;
	default: 
	  _xcp_error("Matching error %d\n", rv);
	  pcre_free(re);
	  return CS_FAIL;
	  break;
	}
      } else {
	if(!cs_parser_getsubstring(re,levent.result,ovector,CS_REGEX_RESULTSIZE*noresult,"number",namedvalue)){
	  if(!sscanf(namedvalue,"%X",&number)){
	    _xcp_error("Failed to parse substring number: %s\n",namedvalue);
	    pcre_free(re);
	    return CS_FAIL;
	  } 
	}
	
	if((number >= daq->ev_max) || (number > CS_XCP_A2L_EVENT_MAX)){
	  _xcp_error("Invalid event number: %s\n",namedvalue);
	  pcre_free(re);
	  return CS_FAIL;
	} 
	
	
	if(!cs_parser_getsubstring(re,levent.result,ovector,CS_REGEX_RESULTSIZE*noresult,"name",namedvalue)){
	  strncpy(daq->event[number].name,namedvalue,CS_PARSER_CHARBUF);
	}
	
	if(!cs_parser_getsubstring(re,levent.result,ovector,CS_REGEX_RESULTSIZE*noresult,"shortname",namedvalue)){
	  strncpy(daq->event[number].shortname,namedvalue,CS_PARSER_CHARBUF);
	}
	
	//Define search area
	llocal.searchin = levent.result;
	llocal.searchlen = levent.resultlen;
	
	if(!cs_a2l_getflag(llocal,"DAQ")){
	  daq->event[number].DAQ_supported = 1;
	}
	
	if(!cs_a2l_getflag(llocal,"STIM")){
	  daq->event[number].STIM_supported = 1;
	}
	
	if(!cs_a2l_getflag(llocal,"DAQ_STIM")){
	  daq->event[number].DAQ_supported = 1;
	  daq->event[number].STIM_supported = 1;
	}
	
	if(!cs_parser_getsubstring(re,levent.result,ovector,CS_REGEX_RESULTSIZE*noresult,"maxdaq",namedvalue)){
	  if(!sscanf(namedvalue,"%X",&lvalue)){
	    _xcp_error("Failed to parse substring maxdaq: %s\n",namedvalue);
	    pcre_free(re);
	    return CS_FAIL;
	  } 
	  daq->event[number].max_daq_list = (int)lvalue;
	}
	
	if(!cs_parser_getsubstring(re,levent.result,ovector,CS_REGEX_RESULTSIZE*noresult,"cycle",namedvalue)){
	  if(!sscanf(namedvalue,"%X",&lvalue)){
	    _xcp_error("Failed to parse substring cycle: %s\n",namedvalue);
	    pcre_free(re);
	    return CS_FAIL;
	  } 
	  daq->event[number].period_units = (uint8_t)lvalue;
	}

	if(!cs_parser_getsubstring(re,levent.result,ovector,CS_REGEX_RESULTSIZE*noresult,"unit",namedvalue)){
	  if(!sscanf(namedvalue,"%X",&lvalue)){
	    _xcp_error("Failed to parse substring unit: %s\n",namedvalue);
	    pcre_free(re);
	    return CS_FAIL;
	  } 
	  daq->event[number].ts_unit = (cs_a2l_ts_unit_t)lvalue;
	}

	daq->event[number].period = cs_cu_ts_unit2double(daq->event[number].ts_unit,daq->event[number].period_units);

	if(!cs_parser_getsubstring(re,levent.result,ovector,CS_REGEX_RESULTSIZE*noresult,"prior",namedvalue)){
	  if(!sscanf(namedvalue,"%X",&lvalue)){
	    _xcp_error("Failed to parse substring prior: %s\n",namedvalue);
	    pcre_free(re);
	    return CS_FAIL;
	  }
	  daq->event[number].priority = (uint8_t)lvalue;
	} 

      }
    }
    if(levent.next == NULL){ //Final entry, no more entries to search
      break;
    } else {
      levent.searchin = levent.next;
      levent.searchlen = levent.nextlen; 
    }
  }

  pcre_free(re); 

  return CS_OK;
}

/**************************************************************************/
int cs_xcp_a2l_GetProtocoll(
			    cs_xcp_a2l_t *data,
			    cs_xcp_a2l_commode_t *mode
			    )
/**************************************************************************/
{
  pcre *re = NULL;
  char regex[CS_REGEX_BUFFER*3];
  int rv = 0;
  int noresult = 13;
  int ovector[CS_REGEX_RESULTSIZE*noresult];
  char namedvalue[CS_PARSER_CHARBUF];
  cs_parser_result_t lresult;
  uint8_t rvalue = 0;

  int i;
  char tname[3];

  if((data == NULL) ||
     (mode == NULL)){
    _xcp_error("Parameter failure\n");
    return CS_FAIL;
  }
  
  //reset commode
  cs_xcp_a2l_data_reset_commode(mode);

  //Build up regex
  memset(regex,0,sizeof(char)*CS_REGEX_BUFFER*3);
  snprintf(regex,CS_REGEX_BUFFER*3,"0x(?'pversion'[0-9a-fA-F]+)[\\s]+0x(?'t1'[0-9a-fA-F]+)[\\s]+0x(?'t2'[0-9a-fA-F]+)[\\s]+0x(?'t3'[0-9a-fA-F]+)[\\s]+0x(?'t4'[0-9a-fA-F]+)[\\s]+0x(?'t5'[0-9a-fA-F]+)[\\s]+0x(?'t6'[0-9a-fA-F]+)[\\s]+0x(?'t7'[0-9a-fA-F]+)[\\s]+0x(?'maxcto'[0-9a-fA-F]+)[\\s]+0x(?'maxdto'[0-9a-fA-F]+)[\\s]+(?'byteorder'BYTE_ORDER_MSB_[A-Z]+)[\\s]+(?'AG'ADDRESS_GRANULARITY_[A-Z]+)[\\s]+");

  //Precompile regex
  if((re = cs_parser_precompile((const char *)regex)) == NULL){
    _xcp_error("Regex precompilation failed\n");
    return CS_FAIL;
  }
  //Get data, only one fit is allowed
  rv = pcre_exec(re, NULL, (const char*)data->a2l.protocol.result,data->a2l.protocol.resultlen, 0,PCRE_NOTEMPTY, ovector, CS_REGEX_RESULTSIZE * noresult);

  if(rv < 0) {
    switch(rv) {
    case PCRE_ERROR_NOMATCH:
      _xcp_debug("No match\n"); 
      break;
    default: 
      _xcp_error("Matching error %d\n", rv);
      pcre_free(re);
      return CS_FAIL;
      break;
    }
  } else {

    if(cs_parser_getsubstring(re,data->a2l.protocol.result,ovector,CS_REGEX_RESULTSIZE*noresult,"pversion",namedvalue)){
      _xcp_error("Failed to get substring pversion\n");
      pcre_free(re);
      return CS_FAIL;
    }
    if(!sscanf(namedvalue,"%X",&mode->prot_version)){
      _xcp_error("Failed to parse substring %s\n",namedvalue);
      pcre_free(re);
      return CS_FAIL;
    }

    for(i = 0; i < XCP_A2L_TO_MAX; i++){
      memset(tname,0,sizeof(char)*3);
      snprintf(tname,3,"t%i",i+1);
      if(cs_parser_getsubstring(re,data->a2l.protocol.result,ovector,CS_REGEX_RESULTSIZE*noresult,tname,namedvalue)){
	_xcp_error("Failed to get substring %s\n",tname);
	pcre_free(re);
	return CS_FAIL;
      }
      if(!sscanf(namedvalue,"%X",&mode->timeout[i])){
	_xcp_error("Failed to parse substring %s\n",namedvalue);
	pcre_free(re);
	return CS_FAIL;
      }
      if(mode->timeout[i] < 10)
	mode->timeout[i] = 100;
    }
    
    if(cs_parser_getsubstring(re,data->a2l.protocol.result,ovector,CS_REGEX_RESULTSIZE*noresult,"maxcto",namedvalue)){
      _xcp_error("Failed to get substring maxcto\n");
      pcre_free(re);
      return CS_FAIL;
    }
    if(!sscanf(namedvalue,"%X",&mode->max_CTO)){
      _xcp_error("Failed to parse substring %s\n",namedvalue);
      pcre_free(re);
      return CS_FAIL;
    }

    if(cs_parser_getsubstring(re,data->a2l.protocol.result,ovector,CS_REGEX_RESULTSIZE*noresult,"maxdto",namedvalue)){
      _xcp_error("Failed to get substring maxdto\n");
      pcre_free(re);
      return CS_FAIL;
    }
    if(!sscanf(namedvalue,"%X",&mode->max_DTO)){
      _xcp_error("Failed to parse substring %s\n",namedvalue);
      pcre_free(re);
      return CS_FAIL;
    }

    if(cs_parser_getsubstring(re,data->a2l.protocol.result,ovector,CS_REGEX_RESULTSIZE*noresult,"byteorder",namedvalue)){
      _xcp_error("Failed to get substring byteorder\n");
      pcre_free(re);
      return CS_FAIL;
    }
    if(strcmp(namedvalue,"BYTE_ORDER_MSB_LAST") == 0){
      mode->BO = A2L_BO_INTEL;
    } else {
      mode->BO = A2L_BO_MOTOROLLA;
    }

    if(cs_parser_getsubstring(re,data->a2l.protocol.result,ovector,CS_REGEX_RESULTSIZE*noresult,"AG",namedvalue)){
      _xcp_error("Failed to get substring AG\n");
      pcre_free(re);
      return CS_FAIL;
    }
    //Default is Byte
    mode->AG = A2L_AG_BYTE;
    if(strcmp(namedvalue,"ADDRESS_GRANULARITY_WORD") == 0)
      mode->AG = A2L_AG_WORD;

    if(strcmp(namedvalue,"ADDRESS_GRANULARITY_DORD") == 0)
      mode->AG = A2L_AG_DWORD;

    //Store AG, to use in in signal size calculation
    data->a2l.AG =  mode->AG;


    _xcp_verbose("Protocol version: %X\n",mode->prot_version);
    _xcp_verbose("T1,T2,T3,T4,T5,T6,T7: %i, %i, %i, %i, %i, %i, %i ms\n",
		 mode->timeout[0],
		 mode->timeout[1],
		 mode->timeout[2],
		 mode->timeout[3],
		 mode->timeout[4],
		 mode->timeout[5],
		 mode->timeout[6]);
    _xcp_verbose("Max CTO: %i Byte, Max DTO: %i Byte\n",mode->max_CTO,mode->max_DTO);
    _xcp_verbose("Byteorder %s\n",mode->BO == A2L_BO_MOTOROLLA?"Motorola":"Intel");
    _xcp_verbose("Address granularity %s\n",mode->AG==A2L_AG_BYTE?"Byte":(mode->AG==A2L_AG_WORD?"Word":"DWord"));
  }
    
  pcre_free(re); 

  if(_cs_xcp_a2l_GetOptional(data,mode)){
    return CS_FAIL;
  }

  if(_cs_xcp_a2l_GetComModeSupported(data,mode)){
    return CS_FAIL;
  }

  lresult.searchin = data->a2l.protocol.result;
  lresult.searchlen = data->a2l.protocol.resultlen;
  mode->queque = (uint8_t)cs_a2l_getuintvalue(lresult,"INTERLEAVED",&rvalue);
  if(rvalue){
    mode->IBM = 0;
  } else {
    mode->IBM = 1;
  }
  _xcp_verbose("Interleaved block mode is %s\n",mode->IBM?"available":"not available");
  _xcp_verbose("Queque size:  %i\n",mode->queque);

  if(_cs_xcp_a2l_GetSeedKey(data,mode)){
    return CS_FAIL;
  }

  return CS_OK;
}


/**************************************************************************/
int cs_xcp_a2l_GetPhysInterfaceCAN(
				   cs_xcp_a2l_t *data,
				   cs_xcp_can_t *ifcan
				   )
/**************************************************************************/
{
  cs_parser_result_t lresult;
  uint8_t rvalue = CS_OK;
  uint32_t id;
  int version;
  
  if((data == NULL) ||
     (ifcan == NULL)){
    _xcp_error("Parameter failure\n");
    return CS_FAIL;
  }
  
  //reset commode
  ifcan = cs_xcp_can_init();

  lresult.searchin = data->a2l.iface.result;
  lresult.searchlen = data->a2l.iface.resultlen;
  if(cs_a2l_search_section(&lresult,"XCP_ON_CAN",NULL)){
    _xcp_error("Regex failure\n");
    return CS_FAIL;
  }

  if(lresult.result == NULL){
    _xcp_error("Cannot find Section XCP_ON_CAN\n");
    return CS_FAIL;
  }

  //cs_parser_printresult(&lresult);

  //define serachregion
  lresult.searchin = lresult.result;
  lresult.searchlen = lresult.resultlen;
 
  //Search first numerical string
  version = (int)cs_a2l_getuintvalue(lresult,NULL,&rvalue);
  if(rvalue){
    _xcp_error("Failed to get version\n");
    return CS_FAIL;
  }
  cs_xcp_can_setVersion(ifcan,version);

  id = cs_a2l_getuintvalue(lresult,"CAN_ID_MASTER",&rvalue);
  if(rvalue){
    _xcp_error("Failed to get Master id\n");
    return CS_FAIL;
  }
  cs_xcp_can_setCmdId(ifcan,id);

  id = cs_a2l_getuintvalue(lresult,"CAN_ID_SLAVE",&rvalue);
  if(rvalue){
    _xcp_error("Failed to get Slave id\n");
    return CS_FAIL;
  }
  cs_xcp_can_setRespId(ifcan,id);

  //Optional
  id = cs_a2l_getuintvalue(lresult,"CAN_ID_BROADCAST",&rvalue);
  cs_xcp_can_setBcastId(ifcan,id);
    

  return CS_OK;
}

/**************************************************************************/
int cs_xcp_a2l_GetDAQ(
		      cs_xcp_a2l_t *data,
		      cs_xcp_a2l_daq_proc_t *daq
		      )
/**************************************************************************/
{
  cs_parser_result_t ldaq;
  cs_parser_result_t lresult;
  cs_parser_result_t lstim;
  cs_parser_result_t ltime;
  uint8_t rvalue = CS_OK;
  pcre *re = NULL;
  char regex[CS_REGEX_BUFFER*3];
  int rv = 0;
  int noresult = 15;
  int ovector[CS_REGEX_RESULTSIZE*noresult];
  char namedvalue[CS_PARSER_CHARBUF];
  uint32_t lvalue;

  if((data == NULL) ||
     (daq == NULL)){
    _xcp_error("Parameter failure\n");
    return CS_FAIL;
  }
  
  //reset commode
  cs_xcp_a2l_init_daqproc(daq);


  //Search daq section
  ldaq.searchin = data->a2l.iface.result;
  ldaq.searchlen = data->a2l.iface.resultlen;
  if(cs_a2l_search_section(&ldaq,"DAQ",NULL)){
    _xcp_error("Regex failure\n");
    return CS_FAIL;
  }

  if(ldaq.result == NULL){
    _xcp_error("Cannot find DAQ\n");
    return CS_FAIL;
  }

  //cs_parser_printresult(&ldaq);

  //define serachregion
  lresult.searchin = ldaq.result;
  lresult.searchlen = ldaq.resultlen;
  lstim = lresult;
  ltime = lresult;

  //get basic settings
  //Build up regex
  memset(regex,0,sizeof(char)*CS_REGEX_BUFFER*3);
  snprintf(regex,CS_REGEX_BUFFER*3,"(?'type'[DYNAMIC|STATIC])[\\s]+0x(?'maxdaq'[0-9a-fA-F]+)[\\s]+0x(?'maxev'[0-9a-fA-F]+)[\\s]+0x(?'mindaq'[0-9a-fA-F]+)[\\s]+(?'optim'OPTIMISATION_TYPE_[A-Z0-9_]+)[\\s]+(?'adrext'ADDRESS_EXTENSION_[A-Z0-9_]+)[\\s]+(?'ident'IDENTIFICATION_FIELD_TYPE_[A-Z0-9_]+)[\\s]+(?'gran'GRANULARITY_ODT_ENTRY_SIZE_DAQ_[A-Z0-9_]+)[\\s]+0x(?'maxodt'[0-9a-fA-F]+)[\\s]+(?'ovl'[A-Z0-9_]+)");

  //Precompile regex
  if((re = cs_parser_precompile((const char *)regex)) == NULL){
    _xcp_error("Regex precompilation failed\n");
    return CS_FAIL;
  }
  //Get data, only one fit is allowed
  rv = pcre_exec(re, NULL, (const char*)ldaq.result,ldaq.resultlen, 0,PCRE_NOTEMPTY, ovector, CS_REGEX_RESULTSIZE * noresult);

  if(rv < 0) {
    switch(rv) {
    case PCRE_ERROR_NOMATCH:
      _xcp_debug("No match\n"); 
      break;
    default: 
      _xcp_error("Matching error %d\n", rv);
      pcre_free(re);
      return CS_FAIL;
      break;
    }
  } else {
    if(cs_parser_getsubstring(re,ldaq.result,ovector,CS_REGEX_RESULTSIZE*noresult,"type",namedvalue)){
      _xcp_error("Failed to get substring type\n");
      pcre_free(re);
      return CS_FAIL;
    }
    if(strcmp(namedvalue,"DYNAMIC") == 0){
      daq->daq_static = 0;
    } else {
      daq->daq_static = 1;
    }

    if(cs_parser_getsubstring(re,ldaq.result,ovector,CS_REGEX_RESULTSIZE*noresult,"maxdaq",namedvalue)){
      _xcp_error("Failed to get substring maxdaq\n");
      pcre_free(re);
      return CS_FAIL;
    }
    if(!sscanf(namedvalue,"%X",&lvalue)){
      _xcp_error("Failed to parse substring %s\n",namedvalue);
      pcre_free(re);
      return CS_FAIL;
    }
    daq->daq_max = (uint16_t)lvalue;
    if(daq->daq_max == 0){
      _xcp_info("ALERT override daq_max value\n");
      daq->daq_max = 0xff;
    }

    if(cs_parser_getsubstring(re,ldaq.result,ovector,CS_REGEX_RESULTSIZE*noresult,"maxev",namedvalue)){
      _xcp_error("Failed to get substring maxev\n");
      pcre_free(re);
      return CS_FAIL;
    }
    if(!sscanf(namedvalue,"%X",&lvalue)){
      _xcp_error("Failed to parse substring %s\n",namedvalue);
      pcre_free(re);
      return CS_FAIL;
    }
    daq->ev_max = (uint16_t)lvalue;

    if(cs_parser_getsubstring(re,ldaq.result,ovector,CS_REGEX_RESULTSIZE*noresult,"mindaq",namedvalue)){
      _xcp_error("Failed to get substring maxev\n");
      pcre_free(re);
      return CS_FAIL;
    }
    if(!sscanf(namedvalue,"%X",&lvalue)){
      _xcp_error("Failed to parse substring %s\n",namedvalue);
      pcre_free(re);
      return CS_FAIL;
    }
    daq->daq_min = (uint8_t)lvalue;

    //Set default
    daq->odt_opt = XCP_A2L_ODT_OPT_DEFAULT;
    if(cs_parser_getsubstring(re,ldaq.result,ovector,CS_REGEX_RESULTSIZE*noresult,"optim",namedvalue)){
      _xcp_error("Failed to get substring optim\n");
      pcre_free(re);
      return CS_FAIL;
    }
    if(strcmp(namedvalue,"OPTIMISATION_TYPE_DEFAULT") == 0){
      daq->odt_opt = XCP_A2L_ODT_OPT_DEFAULT;
    }
    if(strcmp(namedvalue,"OPTIMISATION_TYPE_ODT_TYPE_16") == 0){
      daq->odt_opt = XCP_A2L_ODT_OPT_16;
    }
    if(strcmp(namedvalue,"OPTIMISATION_TYPE_ODT_TYPE_32") == 0){
      daq->odt_opt = XCP_A2L_ODT_OPT_32;
    }
    if(strcmp(namedvalue,"OPTIMISATION_TYPE_ODT_TYPE_64") == 0){
      daq->odt_opt = XCP_A2L_ODT_OPT_64;
    }
    if(strcmp(namedvalue,"OPTIMISATION_TYPE_ODT_TYPE_ALIGNMENT") == 0){
      daq->odt_opt = XCP_A2L_ODT_OPT_AL;
    }
    if(strcmp(namedvalue,"OPTIMISATION_TYPE_MAX_ENTRY_SIZE") == 0){
      daq->odt_opt = XCP_A2L_ODT_OPT_MAX;
    }


    daq->odt_ext = XCP_A2L_ODT_ADDR_EXT_FREE;
    if(cs_parser_getsubstring(re,ldaq.result,ovector,CS_REGEX_RESULTSIZE*noresult,"adrext",namedvalue)){
      _xcp_error("Failed to get substring adrext\n");
      pcre_free(re);
      return CS_FAIL;
    }
    if(strcmp(namedvalue,"ADDRESS_EXTENSION_FREE") == 0){
      daq->odt_ext = XCP_A2L_ODT_ADDR_EXT_FREE;
    }
    if(strcmp(namedvalue,"ADDRESS_EXTENSION_ODT") == 0){
      daq->odt_ext = XCP_A2L_ODT_ADDR_EXT_ODTFIX;
    }
    if(strcmp(namedvalue,"ADDRESS_EXTENSION_DAQ") == 0){
      daq->odt_ext = XCP_A2L_ODT_ADDR_EXT_FIX;
    }

    daq->odt_ident = XCP_A2L_ODT_IDENT_ABS;
    if(cs_parser_getsubstring(re,ldaq.result,ovector,CS_REGEX_RESULTSIZE*noresult,"ident",namedvalue)){
      _xcp_error("Failed to get substring ident\n");
      pcre_free(re);
      return CS_FAIL;
    }
    if(strcmp(namedvalue,"IDENTIFICATION_FIELD_TYPE_ABSOLUTE") == 0){
      daq->odt_ident = XCP_A2L_ODT_IDENT_ABS;
    }
    if(strcmp(namedvalue,"IDENTIFICATION_FIELD_TYPE_RELATIVE_BYTE") == 0){
      daq->odt_ident = XCP_A2L_ODT_IDENT_RAB;
    }
    if(strcmp(namedvalue,"IDENTIFICATION_FIELD_TYPE_RELATIVE_WORD") == 0){
      daq->odt_ident = XCP_A2L_ODT_IDENT_RAW;
    }
    if(strcmp(namedvalue,"IDENTIFICATION_FIELD_TYPE_RELATIVE_WORD_ALIGNED") == 0){
      daq->odt_ident = XCP_A2L_ODT_IDENT_RAWA;
    }

    daq->daq_odt_entry_gr_size = 1; //1Byte
    if(cs_parser_getsubstring(re,ldaq.result,ovector,CS_REGEX_RESULTSIZE*noresult,"gran",namedvalue)){
      _xcp_error("Failed to get substring gran\n");
      pcre_free(re);
      return CS_FAIL;
    }
    if(strcmp(namedvalue,"GRANULARITY_ODT_ENTRY_SIZE_DAQ_BYTE") == 0){
      daq->daq_odt_entry_gr_size = 1;
    }
    if(strcmp(namedvalue,"GRANULARITY_ODT_ENTRY_SIZE_DAQ_WORD") == 0){
      daq->daq_odt_entry_gr_size = 2;
    }
    if(strcmp(namedvalue,"GRANULARITY_ODT_ENTRY_SIZE_DAQ_DWORD") == 0){
      daq->daq_odt_entry_gr_size = 4;
    }
    if(strcmp(namedvalue,"GRANULARITY_ODT_ENTRY_SIZE_DAQ_DLONG") == 0){
      daq->daq_odt_entry_gr_size = 8;
    }

    if(cs_parser_getsubstring(re,ldaq.result,ovector,CS_REGEX_RESULTSIZE*noresult,"maxodt",namedvalue)){
      _xcp_error("Failed to get substring maxodt\n");
      pcre_free(re);
      return CS_FAIL;
    }
    if(!sscanf(namedvalue,"%X",&lvalue)){
      _xcp_error("Failed to parse substring %s\n",namedvalue);
      pcre_free(re);
      return CS_FAIL;
    }
    daq->daq_odt_entry_max_size = (uint8_t)lvalue;


    daq->overload_mode = XCP_A2L_ODT_OVERLOAD_NONE;
    if(cs_parser_getsubstring(re,ldaq.result,ovector,CS_REGEX_RESULTSIZE*noresult,"ovl",namedvalue)){
      _xcp_error("Failed to get substring ovl\n");
      pcre_free(re);
      return CS_FAIL;
    }
    if(strcmp(namedvalue,"NO_OVERLOAD_INDICATION") == 0){
      daq->overload_mode = XCP_A2L_ODT_OVERLOAD_NONE;
    }
    if(strcmp(namedvalue,"OVERLOAD_INDICATION_PID") == 0){
      daq->overload_mode = XCP_A2L_ODT_OVERLOAD_MSB;
    }
    if(strcmp(namedvalue,"OVERLOAD_INDICATION_EVENT") == 0){
      daq->overload_mode = XCP_A2L_ODT_OVERLOAD_EV;
    }
  }

  if(cs_a2l_search_section(&lstim,"STIM",NULL) && (lstim.result != NULL)){
    lstim.searchin = lstim.result;
    lstim.searchlen = lstim.resultlen;
    daq->stim_odt_entry_gr_size = (!cs_a2l_getflag(lstim,"GRANULARITY_ODT_ENTRY_SIZE_STIM_BYTE")?1:daq->stim_odt_entry_gr_size);
    daq->stim_odt_entry_gr_size = (!cs_a2l_getflag(lstim,"GRANULARITY_ODT_ENTRY_SIZE_STIM_WORD")?2:daq->stim_odt_entry_gr_size);
    daq->stim_odt_entry_gr_size = (!cs_a2l_getflag(lstim,"GRANULARITY_ODT_ENTRY_SIZE_STIM_DWORD")?3:daq->stim_odt_entry_gr_size);
    daq->stim_odt_entry_gr_size = (!cs_a2l_getflag(lstim,"GRANULARITY_ODT_ENTRY_SIZE_STIM_DLONG")?4:daq->stim_odt_entry_gr_size);
    lvalue = cs_a2l_getuintvalue(lstim,NULL,&rvalue);
    if(!rvalue){
      daq->stim_odt_entry_max_size = (uint8_t)lvalue;
    }
    daq->bit_stim_supported = !cs_a2l_getflag(lstim,"BIT_STIM_SUPPORTED");
				   
  } else {
    daq->stim_odt_entry_gr_size = 0;
    daq->stim_odt_entry_max_size = 0;
    daq->bit_stim_supported = 0;
  }

  if(cs_a2l_search_section(&ltime,"TIMESTAMP_SUPPORTED",NULL) && (ltime.result != NULL)){
    daq->ts_supported = 1;
    ltime.searchin = ltime.result;
    ltime.searchlen = ltime.resultlen;

    daq->ts_size = (!cs_a2l_getflag(ltime,"NO_TIME_STAMP")?XCP_A2L_TS_SZ_NONE:daq->ts_size);
    daq->ts_size = (!cs_a2l_getflag(ltime,"SIZE_BYTE")?XCP_A2L_TS_SZ_BYTE:daq->ts_size);
    daq->ts_size = (!cs_a2l_getflag(ltime,"SIZE_WORD")?XCP_A2L_TS_SZ_WORD:daq->ts_size);
    daq->ts_size = (!cs_a2l_getflag(ltime,"SIZE_DWORD")?XCP_A2L_TS_SZ_DWORD:daq->ts_size);

    cs_a2l_GetTsUnit(ltime,&daq->ts_unit);

    daq->ts_fixed = !cs_a2l_getflag(ltime,"TIMESTAMP_FIXED");
  } else {
    daq->ts_supported = 0;
    daq->ts_fixed = 0;
  }

 
  //Get optional flags
  daq->prescaler_supported = !cs_a2l_getflag(lresult,"PRESCALER_SUPPORTED");
  daq->resume_supported = !cs_a2l_getflag(lresult,"RESUME_SUPPORTED");
  daq->pid_off_supported = !cs_a2l_getflag(lresult,"PID_OFF_SUPPORTED");


  _cs_xcp_a2l_GetEvents(data,ldaq,daq);

  cs_xcp_a2l_print_processor_info(daq);
   
  return CS_OK;
}

/**************************************************************************/
void cs_xcp_a2l_init(
		     cs_xcp_a2l_t *data  /**< Working storage structure*/
		     )
/**************************************************************************/
{
  if(data == NULL){
    _xcp_error("Parameter failure\n");
    return;
  }
  cs_a2l_init(&data->a2l);

}

/**************************************************************************/
int cs_xcp_a2l_open(
		    cs_xcp_a2l_t *data,
		    char *filename,
		    char *project,
		    char *module,
		    char *interface
		    )
/**************************************************************************/
{
  if((data == NULL) ||
     (filename == NULL) ||
     (project == NULL) ||
     (module == NULL) ||
     (interface == NULL)){
    _xcp_error("Parameter failure\n");
    return CS_FAIL;
  }

  cs_xcp_a2l_init(data);
  
  if(cs_a2l_open(&data->a2l,filename)){
    _xcp_error("Cannot open file %s\n",filename);
    return CS_FAIL;
  }
  
  //Get Project
  if(cs_a2l_GetProject(&data->a2l,project)){
    _xcp_error("Cannot get Project %s\n",project);
    return CS_FAIL;
  }

  //Get Module
  if(cs_a2l_GetModule(&data->a2l,module)){
    _xcp_error("Cannot get Module %s\n",module);
    return CS_FAIL;
  }

  //Get Module
  if(cs_a2l_GetInterface(&data->a2l,interface)){
    _xcp_error("Cannot get Interface %s\n",interface);
    return CS_FAIL;
  }

  //Get common parameter if exists
  if(cs_a2l_GetModCommon(&data->a2l)){
    _xcp_debug("Cannot handle common data\n");
  }

  return CS_OK;
}

/**************************************************************************/
int cs_xcp_a2l_close(
		     cs_xcp_a2l_t *data
		     )
/**************************************************************************/
{
  if(data == NULL){
    _xcp_error("Parameter failure\n");
    return CS_FAIL;
  }

  if(cs_a2l_close(&data->a2l)){
    return CS_FAIL;
  }
  return CS_OK;
}


/** Reset DAQ processor 
 */
/**************************************************************************/
void cs_xcp_a2l_init_daqproc(
			     cs_xcp_a2l_daq_proc_t *proc
			     )
/**************************************************************************/
{
  if(proc == NULL){
    _xcp_error("Parameter failure\n");
    return;
  }
  
  proc->ts_size = XCP_A2L_TS_SZ_NONE;
  proc->ts_fixed = 0; //Static
  proc->daq_odt_entry_gr_size = 1;
  proc->daq_odt_entry_max_size = 4;
  proc->stim_odt_entry_gr_size = 1;
  proc->stim_odt_entry_max_size = 4;
  proc->overload_mode = XCP_A2L_ODT_OVERLOAD_NONE;
  proc->pid_off_supported = 0;
  proc->daq_static = 0;
  proc->prescaler_supported = 0;
  proc->resume_supported = 0;
  proc->bit_stim_supported = 0;
  proc->ts_supported = 0;
  proc->odt_opt = XCP_A2L_ODT_OPT_DEFAULT;
  proc->odt_ext = XCP_A2L_ODT_ADDR_EXT_FREE;
  proc->odt_ident = XCP_A2L_ODT_IDENT_ABS;
  proc->daq_max = 0;
  proc->daq_min = 0;
  memset(proc->event,0,sizeof(cs_xcp_a2l_event_t) * CS_XCP_A2L_EVENT_MAX);
  proc->ev_max = 0;
}

/**************************************************************************/
cs_xcp_a2l_daq_proc_t cs_xcp_a2l_data_get_processor_info(
							 cs_xcp_a2l_t *data
							 )
/**************************************************************************/
{
  cs_xcp_a2l_daq_proc_t ldata;
  
  memset(&ldata,0,sizeof(cs_xcp_a2l_daq_proc_t));
  
  if(data == NULL){
    _xcp_error("Parameter failure\n");
    return ldata;
  }
  
  return data->daq_proc;
}

/**************************************************************************/
cs_xcp_a2l_commode_t cs_xcp_a2l_data_get_commode(
						 cs_xcp_a2l_t *data
						 )
/**************************************************************************/
{
  cs_xcp_a2l_commode_t ldata;

  memset(&ldata,0,sizeof(cs_xcp_a2l_commode_t));

  if(data == NULL){
    _xcp_error("Parameter failure\n");
    return ldata;
  }

  return data->commode;
}


/**************************************************************************/
void cs_xcp_a2l_data_reset_commode(
				   cs_xcp_a2l_commode_t  *mode/**< XCP A2L stack structure*/
				   )
/**************************************************************************/
{
  int i;
  
  if(mode == NULL){
    _xcp_error("Parameter failure\n");
    return;
  }
  
  mode->BO = A2L_BO_INTEL;
  mode->AG = A2L_AG_BYTE;
  mode->SBM = 0;
  mode->optional = 0;
  mode->IBM = 0;
  mode->MBM = 0;
  mode->max_bs = 0;
  mode->min_st = 0;
  mode->max_CTO = 0;
  mode->max_DTO = 0;
  mode->queque = 0;
  
  for(i = 0; i < XCP_A2L_TO_MAX; i++){
    mode->timeout[i] = 2000; //in ms
  }
  
  //Reset feature list
  memset(mode->optfeature,0,XCP_CMD_CONNECT*sizeof(cs_a2l_feature_t));
  
  mode->prot_version = 0;
  mode->trans_version = 0;
  
  memset(mode->seedkey,0,CS_CHARBUF*sizeof(char));
    
}

/**************************************************************************/
int cs_xcp_a2l_data_set_commode(
				cs_xcp_a2l_t *data,
				cs_xcp_a2l_commode_t *mode
				)
/**************************************************************************/
{
  if((data == NULL)||
     (mode == NULL)){
    _xcp_error("Parameter failure\n");
    return CS_FAIL;
  }
  
  data->commode = *mode;
  
  return CS_OK;
}

/**************************************************************************/
void cs_xcp_a2l_print_PhysInterfaceCAN(
				       cs_xcp_can_t *ifcan
				       )
/**************************************************************************/
{
  
  if(!ifcan){
    _xcp_error("Parameter failure\n");
    return;
  }
  
  _xcp_info("Interface name: %s\n",cs_xcp_can_getName(ifcan));
  _xcp_info("Protocol version: 0x%X\n",cs_xcp_can_getVersion(ifcan));
  _xcp_info("Broadcast CAN id: 0x%X\n",cs_xcp_can_getBcastId(ifcan));
  _xcp_info("Master CAN id (Send cmd to Id): 0x%X\n",cs_xcp_can_getCmdId(ifcan));
  _xcp_info(" Slave CAN id (Receive data from Id): 0x%X\n",cs_xcp_can_getRespId(ifcan));
}
/**************************************************************************/
void cs_xcp_a2l_print_event(
			    cs_xcp_a2l_event_t *event
			    )
/**************************************************************************/
{
  
  if(!event){
    _xcp_error("Parameter failure\n");
    return;
  }
  
  _xcp_info("STIM direction is %s\n",event->STIM_supported?"supported":"not supported");
  _xcp_info(" DAQ direction is %s\n",event->DAQ_supported?"supported":"not supported");
  switch(event->consistency){
  case XCP_A2L_EVENT_CONSISTENCY_ODT:
    _xcp_info(" Cosistency level ODT\n");
    break;
  case XCP_A2L_EVENT_CONSISTENCY_DAQ:
    _xcp_info("Cosistency level DAQ\n");
    break;
  case XCP_A2L_EVENT_CONSISTENCY_EVENT:
    _xcp_info("Cosistency level EVENT\n");
    break;
  }
  _xcp_info("Max number of DAQ: %i\n",event->max_daq_list);
  _xcp_info("Name: %s\n",event->name);
  _xcp_info("Short Name: %s\n",event->shortname);
  _xcp_info("Name length: %i\n",event->name_length);
  cs_a2l_print_ts(event->ts_unit,"Event:");
  _xcp_info("Period in units: %i\n", event->period_units);
  _xcp_info("Precalculated Period: %f s\n", event->period);
  _xcp_info("Priority: %X\n",event->priority);
}

/**************************************************************************/
void cs_xcp_a2l_print_processor_info(
				     cs_xcp_a2l_daq_proc_t *daq_proc
				     )
/**************************************************************************/
{
  int i;
  
  if(daq_proc == NULL){
    _xcp_error("Parameter failure\n");
    return;
  }
  
  _xcp_info("DAQ configuration %s\n",daq_proc->daq_static?"dynamic":"static");
  _xcp_info("Prescaler %s\n",daq_proc->prescaler_supported?"available":"not available");
  _xcp_info("Resume %s\n",daq_proc->resume_supported?"available":"not available");
  _xcp_info("Bit STIM %s\n",daq_proc->bit_stim_supported?"available":"not available");
  _xcp_info("Timestamp %s\n",daq_proc->ts_supported?"available":"not available");
  _xcp_info("PID off %s\n",daq_proc->pid_off_supported?"available":"not available");
  switch(daq_proc->overload_mode){
  case XCP_A2L_ODT_OVERLOAD_NONE:
    _xcp_info("Overload signal No Overload signal supported\n");
    break;
  case XCP_A2L_ODT_OVERLOAD_MSB:
    _xcp_info("Overload signal Overload indicated by MSB in PID\n");
    break;
  case XCP_A2L_ODT_OVERLOAD_EV:
    _xcp_info("Overload signal Overload indicated by event package\n");
    break;
  default:
    _xcp_info("Overload signal Unknown\n");
    break;
  }
  _xcp_info("DAQ Max %i\n",daq_proc->daq_max);
  _xcp_info("Event Max %i\n",daq_proc->ev_max);
  _xcp_info("DAQ Min %i\n",daq_proc->daq_min);
  switch(daq_proc->odt_ident){
  case XCP_A2L_ODT_IDENT_ABS:
    _xcp_info("ODT identification field Absolute\n");
    break;
  case XCP_A2L_ODT_IDENT_RAB:
    _xcp_info("ODT identification field Relative ODT, absolute DAQ list number (Byte)\n");
    break;
  case XCP_A2L_ODT_IDENT_RAW:
    _xcp_info("ODT identification field Relative ODT, absolute DAQ list number (Word)\n");
    break;
  case XCP_A2L_ODT_IDENT_RAWA:
    _xcp_info("ODT identification field Relative ODT, absolute DAQ list number (Word, aligned)\n");
    break;
  default:
    _xcp_info("ODT identification field unknown\n");
    break;
  }
  switch(daq_proc->odt_ext){
  case XCP_A2L_ODT_ADDR_EXT_FREE:
    _xcp_info("ODT Address extension Address extension can be different within one and the same ODT\n");
    break;
  case XCP_A2L_ODT_ADDR_EXT_ODTFIX:
    _xcp_info("ODT Address extension Address extension has to be the same within one ODT\n");
    break;
  case XCP_A2L_ODT_ADDR_EXT_FIX:
    _xcp_info("ODT Address extension Address extension has to be the same for all DAQ\n");
    break;
  default:
    _xcp_info("ODT Address extension Unknown\n");
    break;
  }
  switch(daq_proc->odt_opt){
  case XCP_A2L_ODT_OPT_DEFAULT:
    _xcp_info("Optimisation Default\n");
    break;
  case XCP_A2L_ODT_OPT_16:
    _xcp_info("Optimisation 16 Bit\n");
    break;
  case XCP_A2L_ODT_OPT_32:
    _xcp_info("Optimisation 32 Bit\n");
    break;
  case XCP_A2L_ODT_OPT_64:
    _xcp_info("Optimisation 64 Bit\n");
    break;
  case XCP_A2L_ODT_OPT_AL:
    _xcp_info("Optimisation Alignmend\n");
    break;
  case XCP_A2L_ODT_OPT_MAX:
    _xcp_info("Optimisation Max Entry Size\n");
    break;
  default:
    _xcp_info("Optimisation Unknown\n");
    break;
  }
  _xcp_info("DAQ Granuality entry size %i\n",daq_proc->daq_odt_entry_gr_size);
  _xcp_info("ODT Max entry size %i\n",daq_proc->daq_odt_entry_max_size);
  _xcp_info("STIM Granuality entry size %i\n",daq_proc->stim_odt_entry_gr_size);
  _xcp_info("STIM Max entry size %i\n",daq_proc->stim_odt_entry_max_size);
  cs_a2l_print_ts(daq_proc->ts_unit,"DAQ Processor:");
  _xcp_info("Timestamp fixed %s\n",daq_proc->ts_fixed?"yes":"no");
  _xcp_info("Timestamp size %i\n",daq_proc->ts_size);
  
  if(daq_proc->ev_max <= CS_XCP_A2L_EVENT_MAX){
    for(i = 0; i < daq_proc->ev_max; i++){
      cs_xcp_a2l_print_event(&daq_proc->event[i]);
    }
  }
}

/**************************************************************************/
void cs_xcp_a2l_print_commode(
			      cs_xcp_a2l_commode_t *commode
			      )
/**************************************************************************/
{
  if(commode == NULL){
    _xcp_error("Parameter failure\n");
    return;
  }
  
  _xcp_info(" ByteOrder %s\n",commode->BO == A2L_BO_MOTOROLLA?"Motorolla":"Intel");
  switch(commode->AG){
  case  A2L_AG_BYTE:
    _xcp_info("Address Granulation: Byte\n");
    break;
  case  A2L_AG_WORD:
    _xcp_info("Address Granulation: Word\n");
    break;
  case  A2L_AG_DWORD:
    _xcp_info("Address Granulation: Double Word\n");
    break;
  case  A2L_AG_RESERVED:
    _xcp_info("Address Granulation: Reserved\n");
    break;
  default:
    _xcp_info("Address Granulation: Unknown\n");
    break;
  }
  _xcp_info("Slave Block Mode %s\n",commode->SBM?"available":"not available");
  _xcp_info("Optional features %s\n",commode->optional?"available":"not available");
  
  _xcp_info("MAX_CTO: %i\n",commode->max_CTO);
  _xcp_info("MAX_DTO: %i\n",commode->max_DTO);
  
  _xcp_info("Protocol version: %i\n",commode->prot_version);
  _xcp_info("Transport layer version: %i\n",commode->trans_version);
  
  _xcp_info("Interleaved Block Mode %s\n",commode->IBM?"available":"not available");
  _xcp_info(" Master Block Mode %s\n",commode->MBM?"available":"not available");
  _xcp_info("Master Block Mode Max Block size %i\n",commode->max_bs);
  _xcp_info("Master Block Mode Minimal seperation time %i x100 micro seconds \n",commode->min_st);
  _xcp_info("Interleaved block mode queque size %i \n",commode->queque);
  _xcp_info("Slave driver version hex %X \n",commode->driver_version);
}

/** Get atomic data size
    \retval Data size in bytes
*/
/**************************************************************************/
int cs_xcp_a2l_get_ag_size(
			   cs_xcp_a2l_t *data
			   )
/**************************************************************************/
{
  if(data == NULL){
    _xcp_error("Parameter failure\n");
    return 0;
  }
  
  return cs_a2l_get_ag_bytes( data->commode.AG);
}
