/*  
    Copyright (C) 2018 Andreas Stewering-Bone

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. */
/**
 * @example main_canfd.c
 * @author Andreas Stewering-Bone
 * @date 12 December 2018
 * @brief Test CAN stack specific to CAN FD capapilities.
 *
 * Test CAN stack specific to CAN FD capapilities.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <getopt.h>
#include <unistd.h>
#include <time.h>

#include "cs_socketcan.h"


void help(void)
{
  printf(
          "Usage: testcanfd [OPTION]\n"
          "Test library CAN FD stack.\n"
          "\n"
          "Options:\n"
	  "  -i, --interface  CAN Interface\n"
	  "  -v  --verbose    Enable debugging\n"
          "      --help       display this help and exit\n");
}

int main(int argc, char **argv)
{
  int c;
  char * interface = NULL;
  cs_socketcan_t *data;
  int debug = 0;
  int rv;
  
  while (1) {
    static struct option long_options[] = {
      /* These options set a flag. */
      {"interface", required_argument, NULL, 'i'},
      {"help",    no_argument, NULL, 'h'},
      {"verbose",  no_argument, NULL, 'v'},
      {0, 0, 0, 0}
    };
    /* getopt_long stores the option index here. */
    int option_index = 0;
    
    c = getopt_long (argc, argv, "i:hv",
                     long_options, &option_index);
    
    /* Detect the end of the options. */
    if (c == -1)
      break;
    
    switch (c) {
    case 0:
      /* If this option set a flag, do nothing else now. */
      if (long_options[option_index].flag != 0)
        break;
      printf ("option %s", long_options[option_index].name);
      if (optarg)
        printf (" with arg %s", optarg);
      printf ("\n");
      break;
    case 'i': interface = optarg;     break;
    case 'v': debug++;     break;
    case 'h': help(); exit(0);    break;
    case '?':
      /* getopt_long already printed an error message. */
      printf( "Typ `testcanfd --help' for more information\n");
      return -1;
      break;
    default:
      return -1;
      break;
    }
  }
  
  if(interface == NULL){
    printf("Please check arguments\n");
    return -1;
  }


    if(debug == 1){
    cs_socketcan_debuglevel(CS_DEBUG_LEVEL_DEBUG);
  }
  if( debug > 1){
    cs_socketcan_debuglevel(CS_DEBUG_LEVEL_VERBOSE);
  }

  rv = cs_socketcan_checkcompat();
  if(rv < 0){
    printf("Failed to get compability information\n");
    return -1;
  }

  if(rv == 0){
    printf("Detect old libc header, and old kernel\n");
    return -1;
  }

  if(rv == 1){
    printf("Detect old libc header, but new kernel\n");
    return -1;
  }

  if(rv == 2){
    printf("Detect new libc header, but old kernel\n");
    return -1;
  }
  
  if(rv == 3){
    printf("New header and kernel version detected\n");
  }
  
  //Check interface for CAN_FD
  rv = cs_socketcan_check_canfd(interface);
  if(rv < 0){
    printf("Interface failure\n");
    return -1;
  }

  if(rv ==  0){
    printf("Interface has no CAN-FD capabilities\n");
    return -1;
  }

  if(rv == 1){
    printf("Interface is CAN-FD ready\n");
    return -1;
  }
  

  if((data = cs_socketcan_init()) == NULL){
    printf("Failed to init structure\n");
    return -1;
  }
  
  cs_socketcan_free(data);
  return 0;
}
