/*  
    Copyright (C) 2015 Andreas Stewering-Bone

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. */
/**
 * @example main_ccpa2l.c
 * @author Andreas Stewering-Bone
 * @date 5 Jun 2015
 * @brief Simple CCP A2L file parsing application for testing the CCP A2L parsing API.
 *
 * Simple CCP A2L file parsing application for testing the CCP A2L parsing API
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>
#include <unistd.h>
#include <time.h>

#include "cs_ccp.h"


void help(void)
{
  printf(
          "Usage: testccpa2l [OPTION]\n"
          "Test CCP A2L parser stack.\n"
          "\n"
          "Options:\n"
          "  -i, --input    A2L input file\n"
          "  -p, --project  A2L Project\n"
          "  -m, --project  A2L Module\n"
          "  -o, --option   Optional A2L MOD_PAR section\n"
          "      --help     display this help and exit\n");
}

int main(int argc, char **argv)
{
  int c;
  char * filename = NULL;
  char * project = NULL;
  char * module = NULL;
  char * option = NULL;
  cs_ccp_a2l_t data;

  while (1) {
    static struct option long_options[] = {
      /* These options set a flag. */
      {"input", required_argument, NULL, 'i'},
      {"project", required_argument, NULL, 'p'},
      {"module", required_argument, NULL, 'm'},
      {"option", required_argument, NULL, 'o'},
      {"help",    no_argument, NULL, 'h'},
      {0, 0, 0, 0}
    };
    /* getopt_long stores the option index here. */
    int option_index = 0;

    c = getopt_long (argc, argv, "i:p:m:o:h",
                     long_options, &option_index);

    /* Detect the end of the options. */
    if (c == -1)
      break;

    switch (c) {
    case 0:
      /* If this option set a flag, do nothing else now. */
      if (long_options[option_index].flag != 0)
        break;
      printf ("option %s", long_options[option_index].name);
      if (optarg)
        printf (" with arg %s", optarg);
      printf ("\n");
      break;
    case 'i': filename = optarg;     break;
    case 'p': project = optarg;     break;
    case 'm': module = optarg;     break;
    case 'o': option = optarg;     break;
    case 'h': help(); exit(0);    break;
    case '?':
      /* getopt_long already printed an error message. */
      printf( "Typ `testccpa2l --help' for more information\n");
      return -1;
      break;
    default:
      return -1;
      break;
    }
  }

  //First check Library version
  if(CS_CCP_CHECKVERSION){
    printf("Incompatible Library interface, Application was compiled with version %i, Library has version %i\n",
	   CS_CCP_VERSION_MAGIC,cs_ccp_getversion());
    return -1;
  }

  /*Check inputs*/
  if((filename==NULL)||(project==NULL)||(module==NULL)){
    printf( "Please check input arguments\n");
    printf( "Typ `testccpa2l --help' for more information\n");
    return -1;
  }

  //Try to open file
  if(cs_ccp_a2l_open(&data,filename,project,module,"ASAP1B_CCP",option)){
    printf("Failed to open file %s\n",filename);
    return -1;
  }
  cs_ccp_a2l_print(&data);
  //Close file
  if(cs_ccp_a2l_close(&data)){
    printf("Failed to close file\n");
    return -1;
  }

  return 0;
}
