/*  
    Copyright (C) 2018 Andreas Stewering-Bone

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. */
/**
 * @example main_linldf_parser.c
 * @author Andreas Stewering-Bone
 * @date 20 April 2018
 * @brief Simple ldf parser test programm.
 *
 * Simple ldf parser test programm.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>
#include <unistd.h>
#include <time.h>

#include "cs_lin.h"

int verbose = 0;


void help(void)
{
  printf(
	 "Usage: testlinldf_parser [OPTION]\n"
	 "Test lin LDF parser stack.\n"
	 "\n"
	 "Options:\n"
	 " -f   --file <name>   LDF-Filename\n"
	 " -v   --verbose  Verbose output\n"
	 "      --help     display this help and exit\n");
}


int main(int argc, char **argv)
{
  int c;
  cs_lin_ldf_doc_t *doc = NULL;
  char *filename = NULL;
  
  while (1) {
    static struct option long_options[] = {
      /* These options set a flag. */
      {"file",  required_argument, NULL, 'f'},
      {"help",    no_argument, NULL, 'h'},
      {"verbose",    no_argument, NULL, 'v'},
      {0, 0, 0, 0}
    };
    /* getopt_long stores the option index here. */
    int option_index = 0;

    c = getopt_long (argc, argv, "f:hv",
                     long_options, &option_index);

    /* Detect the end of the options. */
    if (c == -1)
      break;

    switch (c) {
    case 0:
      /* If this option set a flag, do nothing else now. */
      if (long_options[option_index].flag != 0)
        break;
      printf ("option %s", long_options[option_index].name);
      if (optarg)
        printf (" with arg %s", optarg);
      printf ("\n");
      break;
    case 'f': filename = optarg;    break;
    case 'v': verbose = 1;    break;
    case 'h': help(); exit(0);    break;
    case '?':
      /* getopt_long already printed an error message. */
      printf( "Typ `testlinldf_parser --help' for more information\n");
      return -1;
      break;
    default:
      return -1;
      break;
    }
  }


  if(filename == NULL){
    printf("No LDF file defined\n");
    return -1;
  }

  if(verbose == 1){
    cs_lin_debuglevel(CS_DEBUG_LEVEL_VERBOSE);
  } else {
    cs_lin_debuglevel(CS_DEBUG_LEVEL_DEBUG);
  }

  
  if((doc = cs_lin_ldf_doc_init()) == NULL){
    printf("Failed to init storage structure");
    return -1;
  }

  if(cs_lin_ldf_doc_open(doc,filename)){
    printf("Failed to parse LDF file \n");
    return -1;
  }
  
  cs_lin_ldf_doc_free(doc);

  return 0;
}
