/*  
    Copyright (C) 2015 Andreas Stewering-Bone

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. */
/**
 * @example main_lists.c
 * @author Andreas Stewering-Bone
 * @date 7 November 2015
 * @brief Simple linked list test programm.
 *
 * Simple linked list test programm.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>
#include <unistd.h>
#include <time.h>

#include "cs_lists.h"


#define NUMBERS 1234

#define VERBOSE 0

void help(void)
{
  printf(
          "Usage: testlists [OPTION]\n"
          "Test linked list stack.\n"
          "\n"
          "Options:\n"
          "      --help     display this help and exit\n");
}

void *cpf(void *ext)
{
  void *t = NULL;

  t = malloc(sizeof(int));

  if(t == NULL)
    return t;
#if VERBOSE
  printf("Internal alloc %p\n",t);
#endif

  memcpy((int*)t,(int*)ext,sizeof(int));

  return t;
}

void delf(void *intern)
{

#if VERBOSE
  printf("Internal free %p\n",intern);
#endif
  free(intern);
}

int main(int argc, char **argv)
{
  int c;
  int i;
  int n[NUMBERS];

  cs_lists_t list;
  struct cs_lists_entry* elem = NULL;

  while (1) {
    static struct option long_options[] = {
      /* These options set a flag. */
      {"help",    no_argument, NULL, 'h'},
      {0, 0, 0, 0}
    };
    /* getopt_long stores the option index here. */
    int option_index = 0;

    c = getopt_long (argc, argv, "h",
                     long_options, &option_index);

    /* Detect the end of the options. */
    if (c == -1)
      break;

    switch (c) {
    case 0:
      /* If this option set a flag, do nothing else now. */
      if (long_options[option_index].flag != 0)
        break;
      printf ("option %s", long_options[option_index].name);
      if (optarg)
        printf (" with arg %s", optarg);
      printf ("\n");
      break;
    case 'h': help(); exit(0);    break;
    case '?':
      /* getopt_long already printed an error message. */
      printf( "Typ `testlists --help' for more information\n");
      return -1;
      break;
    default:
      return -1;
      break;
    }
  }


  //Init data
  for(i = 0; i< NUMBERS; i++){
    n[i] = i;
  }

  //First test with external data

  if(cs_lists_init(&list,NULL,NULL)){
    printf("Failed to init list \n");
    return -1;
  }

  for(i = 0; i < NUMBERS; i++){
    //Insert element
    if(cs_lists_insert(&list,&n[i])){
      printf("Failed to insert data\n");
      return -1;
    }

    printf("List size: %i\n",cs_lists_size(&list));
  }

  //Loop about list, by index
  for(i = 0; i < cs_lists_size(&list); i++){
    printf("Entry pos: %i, value %i\n",i,*((int*)cs_lists_get_iter(&list,i)));
  }

  //Delete in a loop, by index
  for(i = 0; i < cs_lists_size(&list); i++){
    if(cs_lists_delete_iter(&list,i)){
      printf("Delete entry by iter failed\n");
      return -1;
    }
  }


  if(cs_lists_destroy(&list)){
    printf("Failed to destroy list\n");
    return -1;
  }


  //Second test with internal data

  if(cs_lists_init(&list,cpf,delf)){
    printf("Failed to init list \n");
    return -1;
  }

  for(i = 0; i < NUMBERS; i++){
    //Insert element
    if(cs_lists_insert(&list,(void*)&n[i])){
      printf("Failed to insert data\n");
      return -1;
    }

    printf("List size: %i\n",cs_lists_size(&list));

  }

  //Loop about list, by index
  for(i = 0; i < cs_lists_size(&list); i++){
    printf("Internal Entry pos: %i, value %i\n",i,*((int*)cs_lists_get_iter(&list,i)));
  }

  //Delete in a loop, by index
  for(i = 0; i < cs_lists_size(&list); i++){
    if(cs_lists_delete_iter(&list,i)){
      printf("Delete internal entry by iter failed\n");
      return -1;
    }
  }

  if(cs_lists_destroy(&list)){
    printf("Failed to destroy list\n");
    return -1;
  }

  //Third test with internal data and pointer loop

  if(cs_lists_init(&list,cpf,delf)){
    printf("Failed to init list \n");
    return -1;
  }

  for(i = 0; i < NUMBERS; i++){
    //Insert element
    if(cs_lists_insert(&list,(void*)&n[i])){
      printf("Failed to insert data\n");
      return -1;
    }

    printf("List size: %i\n",cs_lists_size(&list));

  }

  //Loop about list, by index
  for(elem = cs_lists_get_first(&list); elem != cs_lists_get_last(&list); elem = cs_lists_get_next(elem)){
    printf("Internal Entry value %i\n",*((int*)cs_lists_get_data(elem)));
  }

  //Delete in a loop, by index
  for(i = 0; i < cs_lists_size(&list); i++){
    if(cs_lists_delete_iter(&list,i)){
      printf("Delete internal entry by iter failed\n");
      return -1;
    }
  }

  if(cs_lists_destroy(&list)){
    printf("Failed to destroy list\n");
    return -1;
  }


  return 0;
}
