/*
    Copyright (C) 2013 Andreas Stewering-Bone

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. */

/**
 * @example main_protocolxml.c
 * @author Andreas Stewering-Bone
 * @date 9 Oct 2014
 * @brief Testing application for XML protocol write API.
 *
 * Testing application for XML protocol write API.
*/

#include <inttypes.h>
#include <stdio.h>
#include <string.h>
#include <getopt.h>
#include <errno.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>

#include "cs_xml_protocol.h"

#define MAXLIST 1024

void help(void)
{
  printf(
          "Usage: testprotocol [OPTION]\n"
          "Test xml protocol stack.\n"
          "\n"
          "Options:\n"
          "  -o, --output   XML-Protocol file\n"
          "      --help     display this help and exit\n");
}

int main(int argc, char **argv)
{
  int c;
  int i;
  double testf[MAXLIST];
  int testd[MAXLIST];
  uint8_t testh[MAXLIST];

  char *xmlname = NULL;
  cs_xml_doc_t data;

  while (1) {
    static struct option long_options[] = {
      /* These options set a flag. */
      {"output", required_argument, NULL, 'o'},
      {"help",    no_argument, NULL, 'h'},
      {0, 0, 0, 0}
    };
    /* getopt_long stores the option index here. */
    int option_index = 0;
    
    c = getopt_long (argc, argv, "o:h",
                     long_options, &option_index);
    
    /* Detect the end of the options. */
    if (c == -1)
      break;
    
    switch (c) {
    case 0:
      /* If this option set a flag, do nothing else now. */
      if (long_options[option_index].flag != 0)
        break;
      printf ("option %s", long_options[option_index].name);
      if (optarg)
        printf (" with arg %s", optarg);
      printf ("\n");
      break;
    case 'o': xmlname = optarg;     break;
    case 'h': help(); exit(0);    break;
    case '?':
      /* getopt_long already printed an error message. */
      printf( "Typ `testxml --help' for more information\n");
      return -1;
      break;
    default:
      return -1;
      break;
    }
  }

  /*Check inputs*/

  if(xmlname == NULL){
    printf( "Please check input arguments\n");
    printf( "Typ `testprotocol --help' for more information\n");
    return -1;
  }

  /*Init fields*/
  for(i=0;i<MAXLIST;i++){
    testf[i] = drand48();
    testd[i] = rand();
    testh[i] = (uint8_t)(0xFF & rand());
  }

  if(!cs_xml_protocol_open(&data, "/tmp",(const char *)xmlname)){
    if(!cs_xml_protocol_section_start(&data,"Identification",NULL)){

      if(cs_xml_protocol_add_data(&data,
				  "Tester",
				  "Ss","My name is Dummy",
				  "Group","Administrators")){
	printf("Failed to add string element section\n");
	return -1;
      }

      if(cs_xml_protocol_add_data(&data,
				  "Tester2",
				  "Fsd",testf,MAXLIST,
				  "Property1","Is this all",
				  "Property2",2)){
	printf("Failed to add Double element section\n");
	return -1;
      }

      if(cs_xml_protocol_add_data(&data,
				  "Tester3",
				  "Dsdhf",testd,MAXLIST,
				  "Property1","Is this all",
				  "Property2",2,
				  "Property3",0x21,
				  "Property3",0.02341)){
	printf("Failed to add Double element section\n");
	return -1;
      }

      if(cs_xml_protocol_add_data(&data,
				  "Tester3",
				  "Hsdhf",testh,MAXLIST,
				  "Property1","Is this all",
				  "Property2",2,
				  "Property3",0x21,
				  "Property3",0.02341)){
	printf("Failed to add Double element section\n");
	return -1;
      }



      if(cs_xml_protocol_section_stop(&data)){
	printf("Failed to close section\n");
	return -1;
      }
    } else {
    printf("Failed to create section\n");
    return -1;
    }


    if(cs_xml_protocol_close(&data)){
      printf("Failed to close file\n");
      return -1;
    }
  } else {
    printf("Failed to open protocol file\n");
    return -1;
  }

  return 0;
}
