/*  
    Copyright (C) 2014 Andreas Stewering-Bone

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. */
/**
 * @example main_vbf.c
 * @author Andreas Stewering-Bone
 * @date 9 Oct 2014
 * @brief Simple VBF file parsing application for testing the VBF parsing API.
 *
 * Simple VBF file parsing application for testing the VBF parsing API.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>
#include <unistd.h>
#include <time.h>

#include "cs_vbf.h"


void help(void)
{
  printf(
          "Usage: testvbf [OPTION]\n"
          "Test VBF parser stack.\n"
          "\n"
          "Options:\n"
          "  -i, --input    VBF input file\n"
          "      --help     display this help and exit\n");
}

int main(int argc, char **argv)
{
  int c;
  char * filename = NULL;
  cs_vbf_t data;

  while (1) {
    static struct option long_options[] = {
      /* These options set a flag. */
      {"input", required_argument, NULL, 'i'},
      {"help",    no_argument, NULL, 'h'},
      {0, 0, 0, 0}
    };
    /* getopt_long stores the option index here. */
    int option_index = 0;

    c = getopt_long (argc, argv, "i:h",
                     long_options, &option_index);

    /* Detect the end of the options. */
    if (c == -1)
      break;

    switch (c) {
    case 0:
      /* If this option set a flag, do nothing else now. */
      if (long_options[option_index].flag != 0)
        break;
      printf ("option %s", long_options[option_index].name);
      if (optarg)
        printf (" with arg %s", optarg);
      printf ("\n");
      break;
    case 'i': filename = optarg;     break;
    case 'h': help(); exit(0);    break;
    case '?':
      /* getopt_long already printed an error message. */
      printf( "Typ `testvbf --help' for more information\n");
      return -1;
      break;
    default:
      return -1;
      break;
    }
  }

  /*Check inputs*/
  if(filename==NULL){
    printf( "Please check input arguments\n");
    printf( "Typ `testvbf --help' for more information\n");
    return -1;
  }

  //Enable debugging
  cs_vbf_debuglevel(VBF_DEBUG_LEVEL_DEBUG);
  
  //Try to open file
  if(cs_vbf_open(&data,filename)){
    printf("Failed to open file %s\n",filename);
    return -1;
  }
  
  //Printout data
  cs_vbf_print(&data);

  //Close file
  if(cs_vbf_close(&data)){
    printf("Failed to close file\n");
    return -1;
  }

  return 0;
}
