#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
Helper script to compile models on wirth
"""

import sys
import os
import zipfile


#define constants
MATLAB='matlab'
MATLABROOT='/vol/opt/Matlab-2009b'
ETHERLAB_DIR='/usr/share/etherlab'
PDSERV_DIR='/usr'
CANRESTBUS_DIR='/usr'

if len(sys.argv) < 0:
    print("No Model file selected")
    print("Codegeneration call %s name.mdl")
    print("Or only compile call %s name")
    sys.exit(-1)

model = sys.argv[1]

(basename,ext) = os.path.splitext(model)

if ext == '':
    #Skip codegeneration
    pass
else:
    #Generate
    rtwgencall="%s -nodesktop -nosplash -r \"try rtwbuild('%s'); catch ME;  exit; end; exit;\"" %(MATLAB,basename)
    print(rtwgencall)
    rv = os.system(rtwgencall)
    if rv<0:
        print('Codegeneration failed')
        sys.exit(-2)

#Compile
makecall='make -C %s_etl_hrt VERBOSE=1 MATLAB_ROOT=%s ETHERLAB_DIR=%s START_DIR=./ EXTRA_CFLAGS="-g -I%s/include" EXTRA_LDFLAGS="-L%s/lib -L%s/lib -lcanrestbus" -f %s.mk' % (basename,MATLABROOT,ETHERLAB_DIR,PDSERV_DIR,PDSERV_DIR,CANRESTBUS_DIR,basename)
print(makecall)
rv = os.system(makecall)
if rv<0:
    print('make failed')
    sys.exit(-2)    

print('make finished successfully')

#Create version info
versionfile = 'VERSION'
versioncall = 'hg id > %s' %(versionfile)
print(versioncall)
rv = os.system(versioncall)
if rv<0:
    print('version call failed')
    sys.exit(-2)    

print('Version file created successfully')

#Create zip file
#default compression via zlib
zipname = '%s.zip' % (basename)
myzip = zipfile.ZipFile(zipname, 'w',zipfile.ZIP_DEFLATED)
#Add model
myzip.write(model)
#Add executeable
myzip.write(basename)
#Add version file
myzip.write(versionfile)
#Add source directory
srcpath = '%s_etl_hrt' % (basename)
for (archiveDirPath, dirNames, fileNames) in os.walk(srcpath):
    for fileName in fileNames:
        filePath = os.path.join(archiveDirPath, fileName)
        myzip.write(filePath)
myzip.close()
print("Create Zip-File: %s successfully." % (zipname))

sys.exit(0)
