/*----------------------------------------------------------------------------
| File:
|   CAN.C
|
| Project:
|   CAN Transport Layer for XCPsim
|
 ----------------------------------------------------------------------------*/

#include <stdio.h>
#include <assert.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <stdarg.h>
//#include <io.h>
//#include <conio.h>
//#include <dos.h>

// CAN driver
//#include "VCanD.h"

// XCP driver
#include "XcpBasic.h"

// XCPsim
#include "XCPsim.h"

// SocketCAN
#include "cs_socketcan.h"

char *caniface                    = 0;
cs_socketcan_t *cansocket          = NULL;

uint32_t gDtoId                 = 2;
uint32_t gCroId                 = 1;

//VportHandle gPortHandle      = INVALID_PORTHANDLE;
//Vaccess gChannelMask         = 0;
//Vaccess gPermissionMask      = 0;
//VDriverConfig *gDriverConfig = 0;




///////////////////////////////////////////////////////////////////////
// canGetDefaults()
//---------------------------------------------------------------------

void canGetDefaults( void )  
{

}

///////////////////////////////////////////////////////////////////////
// InitDriver ()
//---------------------------------------------------------------------
// initializes the CAN driver

//static Vstatus InitDriver( int hwType, int hwIndex, int hwChannel, int open )
int InitDriver( char *ifname, int open )
{
  uint32_t id = (uint32_t)gCroId;

  cansocket = cs_socketcan_init();

  if(cansocket == NULL){
    printf("Failed to init CAN interface %s\n",ifname);
    return 1;
  }

  if(cs_socketcan_config_native(cansocket,ifname,0)){
    printf("Failed to configure CAN interface %s\n",ifname);
    return 1;
  }
  
  if(cs_socketcan_open(cansocket)){
    printf("Failed to init CAN interface %s\n",ifname);
    return 1;
  }
  
  if(cs_socketcan_set_filter(cansocket,&id,1)){
    printf("Failed to set filter CAN Id\n");
    return 1;
  }
  
  return 0;
}                                                  // end InitDriver ()


///////////////////////////////////////////////////////////////////////
// CleanUp()
//---------------------------------------------------------------------
// close the port and the CAN driver

int CleanUp(void)
{
  cs_socketcan_close(cansocket);
  cs_socketcan_free(cansocket);
  return 0;
}                                                    // ..end CleanUp()


///////////////////////////////////////////////////////////////////////
// canSend()
//---------------------------------------------------------------------
// CAN transmit

int canSend( BYTE len, const BYTEPTR msg ) 
{  
  if (len>8) {
    printf("ERROR: Illegal packetsize %u for CAN",len);
    return 0;
  }

  if (len == 0) {
    printf("ERROR: Illegal packetsize zero for CAN");
    return 0;
  }
  
  if(cs_socketcan_send(cansocket,(uint16_t)gDtoId,(uint8_t*)msg,len,SOCKETCAN_SEND_CAN)){
    printf("Failed to send msg\n");
    return 0;
  }
  XcpSendCallBack();

  return 1;
}


///////////////////////////////////////////////////////////////////////
// canMain()
//---------------------------------------------------------------------
// CAN mainloop

void canMain( void ) {

  int vErr;
  struct timeval txRqTime;
  uint32_t id;
  int len = 8;
  uint8_t buffer[len];

  // Initialize the CAN driver
  vErr = InitDriver(caniface,1);
  if (vErr) goto error;

  // Mainloop
  mainRunning = 1;
  while (mainRunning) {

    // Wait for receive event
    //WaitForSingleObject(h,500);
    len = 8;
    if(!cs_socketcan_recv_block(cansocket,&id,buffer,&len,&txRqTime,1)){//1ms cycle
      /*printf("Receive CRO:\n");
      for (i=0;i<len;i++) printf(" %02X",buffer[i]);
      printf("\n");*/
      XcpCommand((void*)buffer);
      
    }
  };


error:
  CleanUp();
  
}
