/*----------------------------------------------------------------------------
  | File:
  |   XCP_CFG.H
  |
  | Project:
  |   PC Emulation of a ECU using XCP on Ethernet or XCP on CAN
  |
  ----------------------------------------------------------------------------*/

#if defined ( __XCP_CFG_H__ )
#else
#define __XCP_CFG_H__


// General includes
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <inttypes.h>



/*----------------------------------------------------------------------------*/
/* Test instrumentation */

/* Turn on screen logging and assertions */

#define XCP_ENABLE_TESTMODE 
#ifdef XCP_ENABLE_TESTMODE

#define ApplXcpPrint printf
#define XCP_ASSERT(x) if (!(x)) ApplXcpPrint("Assertion failed\n");
#define XCP_PARAMETER_CHECKS

#endif

//#define XCP_ENABLE_DPRAM

// Activate autom. map name detection
#define XCP_ENABLE_VECTOR_MAPNAMES

//Enable Memory access via application
#define XCP_ENABLE_MEM_ACCESS_BY_APPL

/*----------------------------------------------------------------------------*/
/* XCP Transport Layer */
#define XCP_TRANSPORT_LAYER_TYPE_ETHERNET


/*----------------------------------------------------------------------------*/
/* Platform specific definitions */

/* 8-Bit qualifier */
typedef unsigned char  vuint8;
typedef signed char    vsint8;   

/* 16-Bit qualifier */
typedef unsigned short vuint16;
typedef signed short   vsint16;

/* 32-Bit qualifier */
typedef unsigned long  vuint32; 
typedef signed long    vsint32;

/* Byte order */
//#define C_CPUTYPE_BIGENDIAN  /* Motorola */
#define C_CPUTYPE_LITTLEENDIAN /* Intel */

        

/*----------------------------------------------------------------------------*/
/* XCP protocol parameters */


/* XCP message length */
#define kXcpMaxCTO     8      /* Maximum CTO Message Lenght */
#define kXcpMaxDTO     8      /* Maximum DTO Message Lenght */


/*----------------------------------------------------------------------------*/
/* XCP protocol data acquisition parameters */

/* Enable DAQ and set the memory size reserved for DAQ lists */
#define XCP_ENABLE_DAQ
#define kXcpDaqMemSize 256
#define XCP_ENABLE_DAQ_PRESCALER
#define XCP_ENABLE_DAQ_OVERRUN_INDICATION
#define XCP_ENABLE_DAQ_RESOLUTION_INFO 
#define XCP_DISABLE_DAQ_HDR_ODT_DAQ
#define XCP_DISABLE_DAQ_TIMESTAMP
#define XCP_ENABLE_MULDAQ_EVENT
#define XCP_ENABLE_DAQ_PROCESSOR_INFO
//#define XCP_ENABLE_TESTMODE
//#define XCP_ENABLE_DAQ_EVENT_INFO
//#define XCP_ENABLE_DAQ_TIMESTAMP_FIXED
#define XCP_DISABLE_DAQ_TIMESTAMP_FIXED
#define XCP_DISABLE_UNALIGNED_MEM_ACCESS
#define XCP_DISABLE_DAQ_OVERRUN_INDICATION
#define XCP_DISABLE_QUEUE_SAMPLE_ODT 
#define XCP_DISABLE_SEND_QUEUE 
//#define XCP_DISABLE_NO_P2INT_CAST



/*Set number of event channels*/
//#define kXcpMaxEvent 4
//uint8_t kXcpEventDirection[] = {0x4,0x4,0x4,0x4};
//uint8_t kXcpEventNameLength[] = {0,0,0,0};
//char kXcpEventName[][5] = {"EV01","EV02","EV03","EV04"};
//uint8_t kXcpEventCycle[] = {0,100,10,1};
//uint8_t kXcpEventUnit[] = {(6<<4),(6<<4),(6<<4),(6<<4)};


#define kXcpSendQueueMinSize 10 



/* Flush the transmit buffer after a command response */
extern void ApplXcpSendFlush( void );
#define ApplXcpSendFlush ApplXcpSendFlush

/* Enable DAQ Timestamps */ 
extern uint32_t tmTimer(void);
#define kXcpDaqTimestampSize                 DAQ_TIMESTAMP_BYTE
#define kXcpDaqTimestampUnit                 DAQ_TIMESTAMP_UNIT_1MS
#define kXcpDaqTimestampTicksPerUnit         1
#define ApplXcpGetTimestamp()                    (XcpDaqTimestampType)tmTimer()              
#define ApplXcpDaqGetTimestamp()                 (XcpDaqTimestampType)tmTimer()             
                                                  
#undef XCP_ENABLE_DPRAM

#endif


